# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco.IOS IPsec classification rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule,DROP_EVENT,Expression
from noc.fm.rules.classes.ipsec import *
from noc.fm.rules.classes.default import DROP

##
## Cisco.IOS IPsec Phase1 Tunnel Start SNMP
##
class Cisco_IOS_IPsec_Phase1_Tunnel_Start_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Phase1 Tunnel Start SNMP"
    event_class=IPsecPhase1TunnelStart
    preference=1000
    required_mibs=["CISCO-IPSEC-FLOW-MONITOR-MIB"]
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.2\.0\.1$"),   # cikeTunnelStart
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.2\.2\.1\.6\..*",r"^(?P<bin_local_ip>....)"),  # cikePeerLocalAddr
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.2\.2\.1\.7\..*",r"^(?P<bin_remote_ip>....)"), # cikePeerRemoteAddr
        Expression("local_ip","decode_ipv4(bin_local_ip)"),
        Expression("remote_ip","decode_ipv4(bin_remote_ip)"),
    ]
##
## Cisco.IOS IPsec Phase1 Tunnel Stop SNMP
##
class Cisco_IOS_IPsec_Phase1_Tunnel_Stop_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Phase1 Tunnel Stop SNMP"
    event_class=IPsecPhase1TunnelStop
    preference=1000
    required_mibs=["CISCO-IPSEC-FLOW-MONITOR-MIB"]
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.2\.0\.2$"),   # cikeTunnelStop
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.2\.2\.1\.6\..*",r"^(?P<bin_local_ip>....)"),  # cikePeerLocalAddr
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.2\.2\.1\.7\..*",r"^(?P<bin_remote_ip>....)"), # cikePeerRemoteAddr
        Expression("local_ip","decode_ipv4(bin_local_ip)"),
        Expression("remote_ip","decode_ipv4(bin_remote_ip)"),
    ]
##
## Cisco.IOS IPsec Phase2 Tunnel Start SNMP
##
class Cisco_IOS_IPsec_Phase2_Tunnel_Start_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Phase2 Tunnel Start SNMP"
    event_class=IPsecPhase2TunnelStart
    preference=1000
    required_mibs=["CISCO-IPSEC-FLOW-MONITOR-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.2\.0\.7$"),      # cipSecTunnelStart
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.3\.3\.1\.10\.",r"(?P<bin_remote_ip>....)"),      # cipSecEndPtRemoteAddr1
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.3\.3\.1\.4\.",r"(?P<bin_local_ip>....)"),        # cipSecEndPtLocalAddr1
        Expression("local_ip","decode_ipv4(bin_local_ip)"),
        Expression("remote_ip","decode_ipv4(bin_remote_ip)"),
    ]
##
## Cisco.IOS IPsec Phase2 Tunnel Stop SNMP
##
class Cisco_IOS_IPsec_Phase2_Tunnel_Stop_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Phase2 Tunnel Stop SNMP"
    event_class=IPsecPhase2TunnelStop
    preference=1000
    required_mibs=["CISCO-IPSEC-FLOW-MONITOR-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.2\.0\.8$"),      # cipSecTunnelStop
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.4\.3\.2\.1\.12\.\d+",r"(?P<bin_remote_ip>....)"),# cipSecEndPtHistRemoteAddr1
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.171\.1\.4\.3\.2\.1\.6\.\d+",r"(?P<bin_local_ip>....)"),  # cipSecEndPtHistLocalAddr1
        Expression("local_ip","decode_ipv4(bin_local_ip)"),
        Expression("remote_ip","decode_ipv4(bin_remote_ip)"),
    ]
##
## Cisco.IOS IPsec Crypto Map Added SNMP
##
class Cisco_IOS_IPsec_Crypto_Map_Added_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Crypto Map Added SNMP"
    event_class=DROP
    preference=1000
    action=DROP_EVENT
    required_mibs=["CISCO-IPSEC-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.10\.62\.2\.0\.3$"), # cipsCryptomapAdded
        (r"^profile$",r"^Cisco\.IOS$"),
    ]

##
## Cisco.IOS IPsec Crypto Map Deleted SNMP
##
class Cisco_IOS_IPsec_Crypto_Map_Deleted_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Crypto Map Deleted SNMP"
    event_class=DROP
    preference=1000
    action=DROP_EVENT
    required_mibs=["CISCO-IPSEC-MIB"]
    patterns=[
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.10\.62\.2\.0\.4$"), # cipsCryptomapDeleted
        (r"^profile$",r"^Cisco\.IOS$"),
    ]
##
## Cisco.IOS IPsec Invalid SPI SYSLOG
##
class Cisco_IOS_IPsec_Invalid_SPI_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Invalid SPI SYSLOG"
    event_class=IPsecInvalidSPI
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^severity$",r"^4$"),
        (r"^message$",r"%CRYPTO-4-RECVD_PKT_INV_SPI: decaps: rec'd IPSEC packet has invalid spi for destaddr=(?P<dst_ip>.+), prot=50, spi=.*, srcaddr=(?P<src_ip>\S+)$"),
    ]
##
## Cisco.IOS IPsec Invalid SPI SYSLOG SNMP
##
class Cisco_IOS_IPsec_Invalid_SPI_SYSLOG_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS IPsec Invalid SPI SYSLOG SNMP"
    event_class=IPsecInvalidSPI
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.2\.0\.1$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.2\.\d+$",r"^CRYPTO$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.4\.\d+$",r"^RECVD_PKT_INV_SPI$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.5\.\d+$",r"^decaps: rec'd IPSEC packet has invalid spi for destaddr=(?P<dst_ip>\S+), prot=50, spi=.+, srcaddr=(?P<src_ip>\S+)$"),
    ]




from south.db import db
from noc.ip.models import *

class Migration:
    
    def forwards(self):
        db.execute(SQL)
    
    def backwards(self):
        db.execute("DROP FUNCTION free_ip(INTEGER,CIDR)")

SQL="""CREATE OR REPLACE
FUNCTION free_ip(INTEGER,CIDR)
RETURNS INET
AS
$$
DECLARE
    p_vrf_id ALIAS FOR $1;
    p_prefix ALIAS FOR $2;
    r        RECORD;
    prev_ip  INET;
BEGIN
    prev_ip=host(network(p_prefix))::inet;
    
    FOR r IN
        SELECT ip
        FROM ip_ipv4address
        WHERE vrf_id=p_vrf_id AND ip << p_prefix AND ip>prev_ip
        ORDER BY ip
    LOOP
        IF r.ip-prev_ip>1 THEN
            RETURN prev_ip+1;
        ELSE
            prev_ip:=r.ip;
        END IF;
    END LOOP;
    IF host(broadcast(p_prefix))::inet - prev_ip > 1 THEN
        RETURN prev_ip+1;
    ELSE
        RETURN NULL;
    END IF;
END;
$$ LANGUAGE plpgsql;
"""



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column,BooleanColumn
import noc.main.report
from noc.fm.models import MIB

class Report(noc.main.report.Report):
    name="fm.unclassified_trap_oids"
    title="Unclassified Trap OIDs"
    requires_cursor=True
    columns=[
        Column("OID"),
        Column("Name"),
        Column("Count",align="RIGHT")]
    
    def get_queryset(self):
        return [(x[0],MIB.get_name(x[0]),x[1]) for x in self.execute("""
            SELECT ed.value,COUNT(*)
            FROM fm_eventdata ed JOIN fm_event e ON (e.id=ed.event_id)
            WHERE ed.key='1.3.6.1.6.3.1.1.4.1.0'
                AND e.event_class_id IN (
                    SELECT id FROM fm_eventclass
                    WHERE name IN ('DEFAULT','SNMP Trap')
                    )
            GROUP BY 1
            ORDER BY 2 DESC""")]




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.cm.models import *

OBJECT_TYPES=["config","dns","prefixlist","rpsl"]
OBJECT_TYPE_CHOICES=[(x,x) for x in OBJECT_TYPES]

class Migration:
    
    def forwards(self):
        db.delete_column("cm_objectcategory","notify_immediately")
        db.delete_column("cm_objectcategory","notify_delayed")
        
        db.create_table('cm_objectlocation', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('name', models.CharField("Name",max_length=64,unique=True)),
            ('description', models.CharField("Description",max_length=128,null=True,blank=True))
        ))
        
        ObjectLocation = db.mock_model(model_name='ObjectLocation', db_table='cm_objectlocation', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        db.execute("INSERT INTO cm_objectlocation(name,description) values(%s,%s)",["default","default location"])
        loc_id=db.execute("SELECT id FROM cm_objectlocation WHERE name=%s",["default"])[0][0]
        
        for ot in OBJECT_TYPES:
            db.add_column("cm_%s"%ot,"location",models.ForeignKey(ObjectLocation,null=True,blank=True))
            db.execute("UPDATE cm_%s SET location_id=%%s"%ot,[loc_id])
            db.execute("ALTER TABLE cm_%s ALTER location_id SET NOT NULL"%ot)
        
        # Mock Models
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectLocation = db.mock_model(model_name='ObjectLocation', db_table='cm_objectlocation', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'ObjectAccess'
        db.create_table('cm_objectaccess', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('type', models.CharField("Type",max_length=16,choices=OBJECT_TYPE_CHOICES)),
            ('category', models.ForeignKey(ObjectCategory,verbose_name="Category",blank=True,null=True)),
            ('location', models.ForeignKey(ObjectLocation,verbose_name="Location",blank=True,null=True)),
            ('user', models.ForeignKey(User,verbose_name=User))
        ))
        
        # Mock Models
        ObjectCategory = db.mock_model(model_name='ObjectCategory', db_table='cm_objectcategory', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        ObjectLocation = db.mock_model(model_name='ObjectLocation', db_table='cm_objectlocation', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'ObjectNotify'
        db.create_table('cm_objectnotify', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('type', models.CharField("Type",max_length=16,choices=OBJECT_TYPE_CHOICES)),
            ('category', models.ForeignKey(ObjectCategory,verbose_name="Category",blank=True,null=True)),
            ('location', models.ForeignKey(ObjectLocation,verbose_name="Location",blank=True,null=True)),
            ('emails', models.CharField("Emails",max_length=128)),
            ('notify_immediately', models.BooleanField("Notify Immediately")),
            ('notify_delayed', models.BooleanField("Notify Delayed")),
        ))
        
        db.send_create_signal('cm', ['ObjectLocation','ObjectAccess','ObjectNotify'])
    
    def backwards(self):
        for ot in OBJECT_TYPES:
            db.delete_column("cm_%s"%ot,"location_id")
        db.delete_table('cm_objectnotify')
        db.delete_table('cm_objectaccess')
        db.add_column("cm_objectcategory","notify_immediately",models.TextField("Notify Immediately",blank=True,null=True))
        db.add_column("cm_objectcategory","notify_delayed",models.TextField("Notify Delayed",blank=True,null=True))
        db.delete_table('cm_objectlocation')
        




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.execute(PROC)
    
    def backwards(self):
        pass

PROC="""
CREATE OR REPLACE
FUNCTION update_event_classification(INTEGER,INTEGER,INTEGER,INTEGER,TEXT,TEXT,TEXT[][])
RETURNS VOID
AS
$$
DECLARE
    p_event_id          ALIAS FOR $1;
    p_event_class_id    ALIAS FOR $2;
    p_event_category_id ALIAS FOR $3; 
    p_event_priority_id ALIAS FOR $4;
    p_subject           ALIAS FOR $5;
    p_body              ALIAS FOR $6;
    p_vars              ALIAS FOR $7;
BEGIN
    DELETE FROM fm_eventdata
    WHERE event_id=p_event_id
        AND "type" IN ('R','V');
    
    FOR i IN array_lower(p_vars,1) .. array_upper(p_vars,1) LOOP
        INSERT INTO fm_eventdata(event_id,"type",key,value)
        VALUES(p_event_id,p_vars[i][1],p_vars[i][2],p_vars[i][3]);
    END LOOP;
    
    UPDATE fm_event
    SET
        event_class_id=p_event_class_id,
        event_category_id=p_event_category_id,
        event_priority_id=p_event_priority_id,
        status='A',
        subject=p_subject,
        body=p_body
    WHERE id=p_event_id;
END;
$$ LANGUAGE plpgsql;
"""


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IHasVlan, IGetVlans
import re

class Script(noc.sa.script.Script):
    name="Generic.has_vlan"
    implements=[IHasVlan]
    requires=[("get_vlans",IGetVlans)]
    def execute(self,vlan_id):
        for v in self.scripts.get_vlans():
            if v["vlan_id"]==vlan_id:
                return True
        return False



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Clean and lightweight non-blocking socket I/O implementation
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
import socket,select,errno,time,logging,pty,os,signal
from errno import EALREADY, EINPROGRESS, EWOULDBLOCK, ECONNRESET, \
     ENOTCONN, ESHUTDOWN, EINTR, EISCONN, ECONNREFUSED, EPIPE, errorcode
from noc.lib.debug import error_report

##
## Abstract non-blocking socket wrapper.
##
class Socket(object):
    TTL=None # maximum time to live in seconds
    def __init__(self,factory,sock):
        self.factory=factory
        self.socket=sock
        self.socket.setblocking(0)
        self.factory.register_socket(self)
        self.name=None
        self.start_time=time.time()
    
    def __del__(self):
        self.debug("Deallocating")
        
    def can_read(self):
        return True
        
    def can_write(self):
        return True
        
    def handle_read(self):
        pass
        
    def handle_write(self):
        pass
        
    def on_close(self):
        pass
        
    def close(self):
        if self.socket:
            self.factory.unregister_socket(self)
            self.socket.close()
            self.socket=None
            self.on_close()
    
    def debug(self,msg):
        logging.debug("[%s(0x%x)] %s"%(self.__class__.__name__,id(self),msg))
    
    def set_name(self,name):
        self.name=name
        self.factory.register_socket(self,name)
    # Stale sockets detection.
    # Called by SocketFactory.close_stale to determine should socket be closed forcefully
    def is_stale(self):
        return self.TTL and time.time()-self.start_time>=self.TTL
##
## Abstract Protocol Parser.
## Accepts data via feed method, polupates internal buffer (self.in_buffer).
## and calls parse_pdu.
## parse_pdu must parse self.in_buffer, retrieve all complete pdu,
## remove them from buffer and return a list of parsed PDU (or empty list if not found)
## All parsed pdu are returned via callback method
class Protocol(object):
    def __init__(self,callback):
        self.callback=callback
        self.in_buffer=""
        
    def feed(self,data):
        self.in_buffer+=data
        for pdu in self.parse_pdu():
            self.callback(pdu)
    
    def parse_pdu(self):
        return []
##
## Line protocol. PDUs are separated by "\n"
##
class LineProtocol(Protocol):
    def on_feed(self):
        pdus=self.in_buffer.split("\n")
        self.in_buffer=pdus.pop(-1)
        return pdus
##
## TCP Listener.
## Waits for connection and creates socket_class instance.
## socket_class should be subclass of AcceptedTCPSocket.
## Should not be used directly. Use SocketFactory.listen_tcp method instead
##
class ListenTCPSocket(Socket):
    def __init__(self,factory,address,port,socket_class):
        Socket.__init__(self,factory,socket.socket(socket.AF_INET,socket.SOCK_STREAM))
        self.socket.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,
            self.socket.getsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR) | 1)
        self.socket.bind((address,port))
        self.socket.listen(5)
        self.socket_class=socket_class
        self.address=address
        self.port=port
    
    def handle_read(self):
        s,addr=self.socket.accept()
        if self.socket_class.check_access(addr[0]):
            self.socket_class(self.factory,s)
        else:
            logging.error("Refusing connection from %s"%addr[0])
            s.close()
##
## UDP Listener
## Waits for UDP packets on specified ports
## and calls on_read for each packet
class ListenUDPSocket(Socket):
    def __init__(self,factory,address,port):
        Socket.__init__(self,factory,socket.socket(socket.AF_INET,socket.SOCK_DGRAM))
        self.socket.setsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR,
            self.socket.getsockopt(socket.SOL_SOCKET,socket.SO_REUSEADDR) | 1)
        self.socket.bind((address,port))
    
    def handle_read(self):
        msg,transport_address=self.socket.recvfrom(8192)
        if msg=="":
            return
        self.on_read(msg,transport_address[0],transport_address[1])
    
    def on_read(self,data,address,port):
        pass
    
    def can_write(self):
        return False

##
## Base class for TCP sockets. Should not be used directly.
## Use ConnectedTCPSocket and AcceptedTCPSocket instead
##
class TCPSocket(Socket):
    protocol_class=None
    def __init__(self,factory,socket):
        Socket.__init__(self,factory,socket)
        self.is_connected=False
        self.out_buffer=""
        if self.protocol_class:
            self.protocol=self.protocol_class(self.on_read)
        self.in_shutdown=False
        
    def close(self,flush=False):
        if flush and len(self.out_buffer)>0:
            self.in_shutdown=True
        else:
            super(TCPSocket,self).close()
        
    def handle_write(self):
        sent=self.socket.send(self.out_buffer)
        self.out_buffer=self.out_buffer[sent:]
        if self.in_shutdown and len(self.out_buffer)==0:
            self.close()
        
    def handle_connect(self):
        self.is_connected=True
        self.on_connect()
        
    def write(self,msg):
        self.debug("write(%s)"%repr(msg))
        self.out_buffer+=msg
        
    def on_read(self,data): pass
    
    def on_connect(self): pass

##
## A socket wrapping accepted TCP connection.
## Following methods can be overrided for desired behavior:
## check_access(cls, address) - called before object creation
## on_connect(self)   - called when socket connected (just after creation)
## on_close(self)     - called when socket closed. Last method called
## on_read(self,data) - called when new portion of data available when protocol=None or when new PDU available
## Following methods used for operation:
## write(self,data)   - send data (Buffered, can be delayed or split into several send)
## close(self)        - close socket (Also implies on_close(self) event)
##
class AcceptedTCPSocket(TCPSocket):
    def __init__(self,factory,socket):
        TCPSocket.__init__(self,factory,socket)
        self.handle_connect()
    
    @classmethod
    def check_access(cls,address):
        return True
        
    def handle_read(self):
        try:
            data=self.socket.recv(8192)
        except socket.error,why:
            if why[0] in (ECONNRESET,ENOTCONN,ESHUTDOWN):
                self.close()
                return
            raise socket.error, why
        if data=="":
            self.close()
            return
        if self.protocol_class:
            self.protocol.feed(data)
        else:
            self.on_read(data)
        
    def can_write(self):
        return self.out_buffer
##
## A socket wrapping connectiong TCP connection.
## Following methods can be overrided for desired behavior:
## on_connect(self)   - called when connection established and socket read to send data (first event)
## on_close(self)     - called when socket closed. Last method called
## on_read(self,data) - called when new portion of data available
## on_conn_refused(self) - called when connection refused (Also implies close(self). first event)
## Following methods used for operation:
## write(self,data)   - send data (Buffered, can be delayed or split into several send)
## close(self)        - close socket (Also implies on_close(self) event)
##
class ConnectedTCPSocket(TCPSocket):
    def __init__(self,factory,address,port,local_address=None):
        TCPSocket.__init__(self,factory,socket.socket(socket.AF_INET,socket.SOCK_STREAM))
        if local_address:
            self.socket.bind((local_address,0))
        e=self.socket.connect_ex((address,port))
        if e in (0, EISCONN):
            #self.handle_read()
            return
        elif e in (EINPROGRESS, EALREADY, EWOULDBLOCK):
            return
        else:
            raise socket.error, (e, errorcode[e])
        
    def handle_read(self):
        if not self.is_connected:
            self.handle_connect()
            return
        try:
            data=self.socket.recv(8192)
        except socket.error,why:
            if why[0]==ECONNREFUSED:
                self.on_conn_refused()
                self.close()
                return
            if why[0] in (ECONNRESET,ENOTCONN,ESHUTDOWN):
                self.close()
                return
            raise socket.error,why
        if not data:
            self.close()
            return
        if self.protocol_class:
            self.protocol.feed(data)
        else:
            self.on_read(data)
    
    def can_write(self):
        return self.out_buffer or not self.is_connected
    
    def handle_write(self):
        if not self.is_connected:
            try:
                self.socket.send("")
            except socket.error,why:
                if why[0]==EPIPE:
                    self.on_conn_refused()
                    self.close()
                    return
                raise socket.error,why
            self.handle_connect()
            return
        TCPSocket.handle_write(self)
    
    def on_conn_refused(self): pass
##
## File wrapper to mimic behavior of socket
##
class FileWrapper(object):
    def __init__(self,fileno):
        self._fileno=fileno
        
    def fileno(self):
        return self._fileno
        
    def recv(self, *args):
        return os.read(self._fileno, *args)
        
    def send(self, *args):
        return os.write(self._fileno, *args)
        
    read = recv
    
    write = send
    
    def close(self):
        os.close(self._fileno)
        
    def setblocking(self,status):
        pass
##
## PTY Socket Emulation
## Events: on_read, on_close
##
class PTYSocket(Socket):
    def __init__(self,factory,argv):
        self.out_buffer=""
        self.pid,fd=pty.fork()
        if self.pid==0:
            os.execv(argv[0],argv)
        else:
            Socket.__init__(self,factory,FileWrapper(fd))
            
    def handle_read(self):
        try:
            data=self.socket.read(8192)
        except OSError:
            self.close()
            return
        if data:
            self.on_read(data)
        else:
            self.close()

    def can_write(self):
        return self.out_buffer
        
    def handle_write(self):
        sent=self.socket.send(self.out_buffer)
        self.out_buffer=self.out_buffer[sent:]

    def handle_connect(self):
        self.is_connected=True
        self.on_connect()

    def write(self,msg):
        self.debug("write(%s)"%repr(msg))
        self.out_buffer+=msg
    
    def close(self):
        Socket.close(self)
        try:
            pid,status=os.waitpid(self.pid,os.WNOHANG)
        except os.error:
            return
        if pid:
            self.debug("Child pid=%d is already terminated. Zombie released"%pid)
        else:
            self.debug("Child pid=%d is not terminated. Killing"%self.pid)
            try:
                os.kill(self.pid,signal.SIGKILL)
            except:
                self.debug("Child pid=%d was killed from another place"%self.pid)
    
    def on_read(self,data):
        pass

##
## Socket Factory.
## Methods:
## listen_tcp  - create new TCP listener
## connect_tcp - create new TCP connection
## run         - main event loop
##
class SocketFactory(object):
    def __init__(self,tick_callback=None):
        self.sockets={}
        self.socket_name={}
        self.name_socket={}
        self.tick_callback=tick_callback
    
    def register_socket(self,socket,name=None):
        logging.debug("register_socket(%s,%s)"%(socket,name))
        self.sockets[socket.socket.fileno()]=socket
        if socket in self.socket_name:
            # Socket was registred
            old_name=self.socket_name[socket]
            del self.socket_name[socket]
            if old_name:
                del self.name_socket[old_name]
        self.socket_name[socket]=name
        if name:
            self.name_socket[name]=socket
        
    def unregister_socket(self,socket):
        logging.debug("unregister_socket(%s)"%socket)
        del self.sockets[socket.socket.fileno()]
        old_name=self.socket_name[socket]
        del self.socket_name[socket]
        if old_name:
            del self.name_socket[old_name]
        
    def listen_tcp(self,address,port,socket_class):
        if not issubclass(socket_class,AcceptedTCPSocket):
            raise "socket_class should be a AcceptedTCPSocket subclass"
        l=ListenTCPSocket(self,address,port,socket_class)
        l.set_name("listen-tcp-%s:%d"%(address,port))
        return l
    
    def connect_tcp(self,address,port,socket_class):
        if not issubclass(socket_class,ConnectedTCPSocket):
            raise "socket_class should be a ConnectedTCPSocket subclass"
        return socket_class(self,address,port)
        
    def get_socket_by_name(self,name):
        return self.name_socket[name]
        
    def get_name_by_socket(self,socket):
        return self.socket_name[socket]
    
    def __len__(self):
        return len(self.sockets)
        
    def close_stale(self):
        for s in [s for s in self.sockets.values() if s.is_stale()]:
            logging.debug("Closing stale socket %s"%s)
            s.close()
        
    def loop(self,timeout=1):
        if self.sockets:
            r=[f for f,s in self.sockets.items() if s.can_read()]
            w=[f for f,s in self.sockets.items() if s.can_write()]
            if r or w:
                try:
                    r,w,x=select.select(r,w,[],timeout)
                except select.error,why:
                    if why[0]==EINTR:
                        return
                    raise
                if r or w:
                    # Write events processed before read events
                    # to catch connection refused causes
                    for f in w:
                        if f in self.sockets:
                            try:
                                self.sockets[f].handle_write()
                            except:
                                error_report()
                                try:
                                    self.sockets[f].close()
                                except:
                                    pass
                    for f in r:
                        if f in self.sockets:
                            try:
                                self.sockets[f].handle_read()
                            except:
                                error_report()
                                try:
                                    self.sockets[f].close()
                                except:
                                    pass
            else:
                time.sleep(timeout)
        else:
            time.sleep(timeout)
    
    def run(self,run_forever=False):
        if run_forever:
            cond=lambda:True
        else:
            cond=lambda:len(self.sockets)>0
            # Wait for any socket
            while not cond():
                time.sleep(1)
        last_tick=time.time()
        last_stale=time.time()
        while cond():
            self.loop(1)
            t=time.time()
            if self.tick_callback and t-last_tick>=1:
                self.tick_callback()
                last_tick=t
            if t-last_stale>10:
                self.close_stale()
                last_stale=t
    
    ##
    ## Return amount of active sockets which are descendants from sclass
    ##
    def count_subclass_sockets(self,sclass):
        return len([s for s in self.sockets.values() if issubclass(s.__class__,sclass)])



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.sa.interfaces.base import *
from noc.sa.interfaces.igetconfig import IGetConfig
from noc.sa.interfaces.igetvlans import IGetVlans
from noc.sa.interfaces.ihasvlan import IHasVlan
from noc.sa.interfaces.igetversion import IGetVersion
from noc.sa.interfaces.igetmacaddresstable import IGetMACAddressTable
from noc.sa.interfaces.igetdot11associations import IGetDot11Associations
from noc.sa.interfaces.iping import IPing




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.sa.models import *
import os

class Migration:
    
    def forwards(self):
        def qget(map,key):
            if key is None:
                return None
            return map[key]
        db.start_transaction()
        # Fill administrative domains
        location2domain={}
        for id,name,description in db.execute("SELECT id,name,description FROM cm_objectlocation"):
            db.execute("INSERT INTO sa_administrativedomain(name,description) VALUES(%s,%s)",[name,description])
            location2domain[id]=db.execute("SELECT id FROM sa_administrativedomain WHERE name=%s",[name])[0][0]
        # Fill groups
        category2group={}
        for id,name,description in db.execute("SELECT id,name,description FROM cm_objectcategory"):
            db.execute("INSERT INTO sa_objectgroup(name,description) VALUES(%s,%s)",[name,description])
            category2group[id]=db.execute("SELECT id FROM sa_objectgroup WHERE name=%s",[name])[0][0]
        #
        ManagedObject = db.mock_model(model_name='ManagedObject', db_table='sa_managedobject', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        db.add_column("cm_config","managed_object",models.ForeignKey(ManagedObject,null=True))
        
        # Move objects
        for id,repo_path,activator_id,profile_name,scheme,address,port,user,password,super_password,remote_path,location_id,trap_source_ip,trap_community\
            in db.execute("SELECT id,repo_path,activator_id,profile_name,scheme,address,port,\"user\",password,super_password,remote_path,location_id,trap_source_ip,trap_community FROM cm_config"):
            name=os.path.basename(repo_path)
            db.execute("INSERT INTO sa_managedobject(name,repo_path,activator_id,profile_name,scheme,address,port,\"user\",password,super_password,remote_path,administrative_domain_id,trap_source_ip,trap_community) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s)",
            [name,repo_path,activator_id,profile_name,scheme,address,port,user,password,super_password,remote_path,location2domain[location_id],trap_source_ip,trap_community])
            new_id=db.execute("SELECT id FROM sa_managedobject WHERE name=%s",[name])[0][0]
            for object_id,objectcategory_id in db.execute("SELECT object_id,objectcategory_id FROM cm_object_categories WHERE object_id=%s",[id]):
                db.execute("INSERT INTO sa_managedobject_groups(manageobject_id,objectgroup_id) VALUES(%s,%s)",[new_id,category2group[objectcategory_id]])
            db.execute("UPDATE cm_config SET managed_object_id=%s WHERE id=%s",[new_id,id])
        # Move user access
        for category_id,location_id,user_id in db.execute("SELECT category_id,location_id,user_id FROM cm_objectaccess"):
            db.execute("INSERT INTO sa_useraccess(user_id,administrative_domain_id,group_id) VALUES(%s,%s,%s)",[user_id,qget(location2domain,location_id),qget(category2group,category_id)])
        db.execute("ALTER TABLE cm_config ALTER managed_object_id SET NOT NULL")
        
        # Migrate ObjectNotify
        db.add_column("cm_objectnotify","administrative_domain",models.ForeignKey(AdministrativeDomain,verbose_name="Administrative Domain",blank=True,null=True))
        db.add_column("cm_objectnotify","group",models.ForeignKey(ObjectGroup,verbose_name="Group",blank=True,null=True))
        for id,category_id,location_id in db.execute("SELECT id,category_id,location_id FROM cm_objectnotify"):
            db.execute("UPDATE cm_objectnotify SET administrative_domain_id=%s,group_id=%s WHERE id=%s",[qget(location2domain,location_id),qget(category2group,category_id),id])
        db.commit_transaction()
    
    def backwards(self):
        "Write your backwards migration here"
        db.delete_column("cm_config","managed_object_id")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

class Migration:
        def forwards(self):
            db.add_column("dns_dnsserver","provisioning",models.CharField("Provisioning",max_length=128,blank=True,null=True))
            db.execute("UPDATE dns_dnsserver SET provisioning=%s",["%(rsync)s -av --delete * /tmp/dns"])

        def backwards(self):
            db.delete_column("dns_dnsserver","provisioning")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Backup database and repo to main.backupdir
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.periodic
from noc.settings import config
import os,subprocess,datetime
import logging

class Task(noc.sa.periodic.Task):
    name="main.backup"
    description=""
    
    def execute(self):
        def safe_unlink(path):
            logging.debug("Unlinking: %s"%path)
            try:
                os.unlink(path)
            except:
                pass
        from django.conf import settings
        # Build backup path
        now=datetime.datetime.now()
        out="noc-%04d-%02d-%02d-%02d-%02d.dump"%(now.year,now.month,now.day,now.hour,now.minute)
        out=os.path.join(config.get("path","backup_dir"),out)
        # Build pg_dump command and options
        cmd=[config.get("path","pg_dump"),"-Fc"]
        cmd+=["-f",out]
        if settings.DATABASE_USER:
            cmd+=["-U",settings.DATABASE_USER]
        #if settings.DATABASE_PASSWORD:
        #    cmd+=["-W"]
        if settings.DATABASE_HOST:
            cmd+=["-h",settings.DATABASE_HOST]
        if settings.DATABASE_PORT:
            cmd+=["-p",str(settings.DATABASE_PORT)]
        cmd+=[settings.DATABASE_NAME]
        # Launch pg_dump
        logging.info("main.backup: dumping database into %s"%out)
        retcode=subprocess.call(cmd)
        if retcode!=0:
            logging.error("main.backup: dump failed. Removing broken dump '%s'"%out)
            safe_unlink(out)
            return False
        #
        # Back up repo
        #
        repo_root=config.get("cm","repo")
        repo_out="noc-%04d-%02d-%02d-%02d-%02d.tar.gz"%(now.year,now.month,now.day,now.hour,now.minute)
        repo_out=os.path.join(config.get("path","backup_dir"),repo_out)
        logging.info("main.backup: dumping repo into %s"%repo_out)
        tar_cmd=[config.get("path","tar"),"cf","-"]+[f for f in os.listdir(repo_root) if not f.startswith(".")]
        gzip_cmd=[config.get("path","gzip")]
        f=open(repo_out,"w")
        p1=subprocess.Popen(tar_cmd,cwd=repo_root,stdout=subprocess.PIPE)
        p2=subprocess.Popen(gzip_cmd,stdin=p1.stdout,stdout=f)
        f.close()
        return True
        #logging.error("main.backup: repo dump failed. Removing broken dumps")
        #safe_unlink(repo_out)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import datetime,os
from noc.lib.registry import Registry

##
## Registry for all periodic tasks
##
class PeriodicRegistry(Registry):
    name="PeriodicRegistry"
    subdir="periodics"
    classname="Task"
    
periodic_registry=PeriodicRegistry()

##
## Metaclass for Task
##
class TaskBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        periodic_registry.register(m.name,m)
        return m
##
## Task handler
##
class Task(object):
    __metaclass__ = TaskBase
    name=None
    description=""
    def __init__(self,sae):
        self.sae=sae
    def execute(self):
        return True
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## noc-fcgi daemon
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.daemon import Daemon
from django.core.handlers.wsgi import WSGIHandler
from flup.server.fcgi import WSGIServer

class FCGI(Daemon):
    daemon_name="noc-fcgi"
    def run(self):
        wsgi_opts={
            "minSpare"   : int(self.config.getint("fcgi","minspare")),
            "maxSpare"   : int(self.config.getint("fcgi","maxspare")),
            "maxThreads" : self.config.getint("fcgi","maxchildren"),
            "debug"      : False, # turn off flup tracebacks
            "bindAddress": self.config.get("fcgi","socket"),
            "umask"      : 0, # Emulate BSD behavior on linux
        }
        WSGIServer(WSGIHandler(), **wsgi_opts).run()




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.add_column("fm_eventclassificationre","is_expression",models.BooleanField("Is Expression",default=False))
    
    def backwards(self):
        db.delete_column("fm_eventclassificationre","is_expression")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## noc-classifier daemon
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.lib.daemon import Daemon
from noc.lib.pyquote import bin_quote,bin_unquote
from noc.lib.validators import is_ipv4
from noc.fm.models import EventClassificationRule,Event,EventData,EventClass,MIB,EventClassVar,EventRepeat,EventPostProcessingRule
from django.db import transaction,reset_queries
from django.template import Template, Context
import re,logging,time,datetime

##
## Patterns
##
rx_template=re.compile(r"\{\{([^}]+)\}\}")
rx_oid=re.compile(r"^(\d+\.){6,}")
rx_mac_cisco=re.compile(r"^[0-9a-f]{4}\.[0-9a-f]{4}\.[0-9a-f]{4}$")
##
## Global functions for variables evaluation
##

##
## Decode IPv4 from 4 octets
##
def decode_ipv4(s):
    if len(s)==4:
        return ".".join(["%d"%ord(c) for c in list(s)])
    if is_ipv4(s):
        return s
    raise DecodeError
##
## Decode MAC address from 6 octets
##
def decode_mac(s):
    if len(s)==6:
        return "%02X:%02X:%02X:%02X:%02X:%02X"%tuple([ord(x) for x in list(s)])
    if rx_mac_cisco.match(s):
        s=s.replace(".","").upper()
        return "%s:%s:%s:%s:%s:%s"%(s[:2],s[2:4],s[4:6],s[6:8],s[8:10],s[10:])
    raise DecodeError
##
## Global variables dict
##
eval_globals={
    "decode_ipv4" : decode_ipv4,
    "decode_mac"  : decode_mac,
}

##
## Exceptions
##
class DecodeError(Exception): pass
##
## In-memory Rule representation
##
class Rule(object):
    def __init__(self,rule):
        self.rule=rule
        self.name=rule.name
        self.re=[(re.compile(x.left_re,re.MULTILINE|re.DOTALL),re.compile(x.right_re,re.MULTILINE|re.DOTALL)) for x in rule.eventclassificationre_set.filter(is_expression=False)]
        self.expressions=[(x.left_re,compile(x.right_re,"inline","eval")) for x in rule.eventclassificationre_set.filter(is_expression=True)]
        if not self.expressions:
            self.expressions=None
    ##
    ## Return a hash of extracted variables for object o, or None
    ##
    def match(self,o):
        vars={}
        for l,r in self.re:
            found=False
            for o_l,o_r in o:
                l_match=l.search(o_l)
                if not l_match:
                    continue
                r_match=r.search(o_r)
                if not r_match:
                    return None
                found=True
                vars.update(l_match.groupdict()) # Populate vars with extracted variables
                vars.update(r_match.groupdict())
                break
            if not found:
                return None
        return vars
    ##
    ## Decode IPv4 from 4 octets
    ##
    def decode_ipv4(self,s):
        if len(s)==4:
            return ".".join(["%d"%ord(c) for c in list(s)])
        if is_ipv4(s):
            return s
        raise DecodeError
    ##
    ## Decode MAC address from 6 octets
    ##
    def decode_mac(self,s):
        if len(s)==6:
            return "%02X:%02X:%02X:%02X:%02X:%02X"%tuple([ord(x) for x in list(s)])
        if rx_mac_cisco.match(s):
            s=s.replace(".","").upper()
            return "%s:%s:%s:%s:%s:%s"%(s[:2],s[2:4],s[4:6],s[6:8],s[8:10],s[10:])
        raise DecodeError
##
## Post-Process rule
##
class PostProcessRule(object):
    def __init__(self,rule):
        self.rule=rule
        self.name=rule.name
        self.re=[(re.compile(x.var_re,re.MULTILINE|re.DOTALL),re.compile(x.value_re,re.MULTILINE|re.DOTALL)) for x in rule.eventpostprocessingre_set.all()]
    
    def match(self,vars):
        v=vars.items()
        for rx_var,rx_value in self.re:
            matched=False
            for var,value in v:
                if rx_var.match(var) and rx_value.match(value):
                    matched=True
                    break
            if not matched:
                return False
        return True

##
## Noc-classifier daemon
##
class Classifier(Daemon):
    daemon_name="noc-classifier"
    def __init__(self):
        self.rules=[]
        self.templates={} # event_class_id -> (body_template,subject_template)
        self.post_process={} # event_class_id -> [rule1, ..., ruleN]
        Daemon.__init__(self)
        logging.info("Running Classifier")
    ##
    ## Load rules from database after loading config
    ##
    def load_config(self):
        super(Classifier,self).load_config()
        self.load_rules()
    ##
    ## Load rules from database
    ##
    def load_rules(self):
        logging.info("Loading rules")
        self.rules=[Rule(r) for r in EventClassificationRule.objects.order_by("preference")]
        logging.info("%d rules are loaded"%len(self.rules))
        logging.info("Compiling templates")
        self.templates=dict([(ec.id,(Template(ec.subject_template),Template(ec.body_template))) for ec in EventClass.objects.all()])
        logging.info("%d templates are compiled"%len(self.templates)*2)
        logging.info("Loading post-process rules")
        self.post_process={}
        n=0
        for r in EventPostProcessingRule.objects.order_by("preference"):
            ec_id=r.event_class.id
            if ec_id not in self.post_process:
                self.post_process[ec_id]=[]
            self.post_process[ec_id].append(PostProcessRule(r))
            n+=1
        logging.info("%d post-processing rules are loaded"%n)
    ##
    ## Classify single event:
    ## 1. Resolve OIDs when source is SNMP Trap
    ## 2. Try to find matching rule
    ## 3. Drop event if required by rule
    ## 4. Set event class of the matched rule or DEFAULT
    ## 
    def classify_event(self,event):
        # Extract received event properties
        props=[(x.key,bin_unquote(x.value)) for x in event.eventdata_set.filter(type=">")]
        # Resolve additional event properties
        resolved={
            "profile":event.managed_object.profile_name
        }
        # Resolve SNMP oids
        if self.get_source(props)=="SNMP Trap":
            resolved.update(self.resolve_snmp_oids(props))
        props+=resolved.items()
        # Find rule
        status="A"
        rule,vars=self.find_classification_rule(event,props)
        if rule: # Rule found
            if rule.rule.action=="D": # Drop event when required by rule
                self.drop_event(event)
                return
            event_class=rule.rule.event_class
            event.event_class=event_class
            if event_class.repeat_suppression and event_class.repeat_suppression_interval>0 and self.suppress_repeat(event,vars):
                return # Event is suppressed, no further processing
            event_category=event_class.category
            event_priority=event_class.default_priority
            # Find post-processing rule
            post_process=self.find_post_processing_rule(event,vars)
            if post_process:
                if post_process.rule.action=="D": # Drop event if required by post_process_rule
                    self.drop_event(event)
                    return
                if post_process.rule.change_category:
                    event_category=post_process.rule.change_category # Set up priority and category from rule
                if post_process.rule.change_priority:
                    event_priority=post_process.rule.change_priority
                status=post_process.rule.action
            event.log("CLASSIFICATION RULE: %s"%rule.name,to_status=status)
            if post_process:
                event.log("POST-PROCESS RULE: %s"%post_process.name,from_status=status,to_status=status)
        else:
            # Set event class to DEFAULT when no matching rule found
            event_class=EventClass.objects.get(name="DEFAULT")
            vars={}
            logging.debug("No rule found for event %d. Falling back to DEFAULT"%event.id)
            event_category=event_class.category
            event_priority=event_class.default_priority
            event.log("FALLBACK TO DEFAULT")
        # Fill event subject and body
        f_vars=dict(props) # f_vars contains all event vars, including original, extracted and resolved
        f_vars.update(vars)
        f_vars.update(resolved)
        # Does rule contain additional expressions which are need to be calculated?
        if rule and rule.expressions:
            c_vars=dict([(x[0],str(eval(x[1],f_vars,eval_globals))) for x in rule.expressions]) # Evaluate all expressions
            f_vars.update(c_vars) # Update var dicts
            vars.update(c_vars)
        subject_template,body_template=self.templates[event_class.id]
        context=Context(f_vars)
        subject=subject_template.render(context)
        if len(subject)>255: # Too long subject must be truncated
            subject=subject[:250]+" ..."
        body=body_template.render(context)
        # Set up event
        event.event_class=event_class
        event.event_category=event_category
        event.event_priority=event_priority
        event.status=status
        event.subject=subject
        event.body=body
        event.save()
        # Write event vars
        event.eventdata_set.filter(type__in=["R","V"]).delete() # Delete old "R" and "V" vars
        # Write vars
        for k,v in resolved.items():
            EventData(event=event,key=k,value=bin_quote(v),type="R").save()
        for k,v in vars.items():
            EventData(event=event,key=k,value=bin_quote(v),type="V").save()
        # Finally run event class trigger
        event.event_class.run_trigger(event)
    ##
    ## Return event source
    ##
    def get_source(self,props):
        for k,v in props:
            if k=="source":
                return v
        return None
    ##
    ## Resolve SNMP oids to symbolic names
    ##
    def resolve_snmp_oids(self,props):
        def is_oid(s):
            return rx_oid.search(s) is not None
        resolved={}
        for k,v in props:
            if is_oid(k):
                oid=MIB.get_name(k)
                if oid!=k:
                    if is_oid(v):
                        v=MIB.get_name(v)
                    resolved[oid]=v
        return resolved
        
    ##
    ## Find classification rule.
    ## Returns Rule,vars or None,None
    ##
    def find_classification_rule(self,event,props):
        for r in self.rules:
            # Try to match rule
            vars=r.match(props)
            if vars is not None:
                logging.debug("Matching class for event %d found: %s (Rule: %s)"%(event.id,r.rule.event_class.name,r.name))
                return r,vars
        return None,None
        
    ##
    ## Find matching postprocessing rule for event class.
    ## Returns PostProcessRule or None
    ##
    def find_post_processing_rule(self,event,vars):
        event_class=event.event_class
        if event_class.id in self.post_process:
            for r in self.post_process[event_class.id]:
                if r.match(vars):
                    logging.debug("Event #%d matches post-processing rule '%s'"%(event.id,r.name))
                    return r
        return None
    ##
    ## Drop event
    ##
    def drop_event(self,event):
        logging.debug("Drop event #%d"%event.id)
        event.delete()
    ##
    ## Suppress repeats.
    ## Return True if event is suppressed, False otherwise
    ##
    def suppress_repeat(self,event,vars):
        event_class=event.event_class
        # Build keys
        kv={}
        for name in [v.name for v in EventClassVar.objects.filter(event_class=event_class,repeat_suppression=True)]:
            if name in vars:
                kv[name]=vars[name]
            else:
                return False
        r=[e for e in Event.objects.filter(
            event_class=event_class,
            managed_object=event.managed_object,
            timestamp__gte=event.timestamp-datetime.timedelta(seconds=event_class.repeat_suppression_interval),
            timestamp__lte=event.timestamp
            ).exclude(id=event.id).order_by("-timestamp")
            if e.match_data(kv)]
        if len(r)>0:
            pe=r[0]
            logging.debug("Event #%d repeats event #%d"%(event.id,pe.id))
            er=EventRepeat(event=pe,timestamp=pe.timestamp)
            er.save()
            pe.timestamp=event.timestamp
            pe.save()
            event.delete()
            return True
    ##
    ## Main daemon loop
    ##
    def run(self):
        CHUNK=1000 # Maximum amount of events to proceed at once
        INTERVAL=10
        transaction.enter_transaction_management()
        while True:
            n=0
            t0=time.time()
            for e in Event.objects.filter(status="U").order_by("-id")[:CHUNK]:
                self.classify_event(e)
                transaction.commit()
                reset_queries() # Free queries log
                n+=1
            if n: # Write out performance data
                dt=time.time()-t0
                if dt>0:
                    perf=n/dt
                else:
                    perf=0
                logging.info("%d events classified (%10.4f second elapsed. %10.4f events/sec)"%(n,dt,perf))
            else: # No events classified this pass. Sleep
                time.sleep(INTERVAL)
        transaction.leave_transaction_management()
        




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Models for KB application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from noc.kb.models import *
from noc.lib.render import render
from noc.main.models import MIMEType
from django.shortcuts import get_object_or_404
from django.http import HttpResponse,HttpResponseNotFound,HttpResponseRedirect
from django import forms

##
## Title page
##
def index(request):
    return index_bookmarks(request)
##
## Bookmarks page
##
def index_bookmarks(request):
    entries=[b.kb_entry for b in KBGlobalBookmark.objects.all()]+[b.kb_entry for b in KBUserBookmark.objects.filter(user=request.user)]
    return render(request,"kb/index.html",
        {
        "tab"     : "bookmarks",
        "entries" : entries,
        })
##
## Latest Changes page
##
def index_latest(request):
    return render(request,"kb/index.html",
        {
        "tab"     : "latest",
        "entries" : KBEntry.last_modified(20),
        })
##
## Popular Articles
##
def index_popular(request):
    return render(request,"kb/index.html",
        {
        "tab"     : "popular",
        "entries" : KBEntry.most_popular(20),
        })
##
## All Pages
##
def index_all(request):
    return render(request,"kb/index.html",
    {
        "tab"     : "all",
        "entries" : KBEntry.objects.order_by("-id")
    })
##
## KB Entry Preview
##
def view(request,kb_id):
    e=get_object_or_404(KBEntry,id=int(kb_id))
    e.log_preview(request.user)
    return render(request,"kb/view.html",{"e":e,"has_bookmark":e.is_bookmarked(request.user)})
##
## Download attachment
##
def attachment(request,kb_id,name):
    e=get_object_or_404(KBEntry,id=int(kb_id))
    a=get_object_or_404(KBEntryAttachment,kb_entry=e,name=name)
    return HttpResponse(a.file.file.read(),content_type=MIMEType.get_mime_type(a.file.name))
##
## Manipulate user bookmark
##     action is one of "set" or "unset"
##
def bookmark(request,kb_id,action):
    if action not in ("set","unset"):
        return HttpResponseNotFound()
    e=get_object_or_404(KBEntry,id=int(kb_id))
    if action=="set":
        e.set_user_bookmark(request.user)
    else:
        e.unset_user_bookmark(request.user)
    return HttpResponseRedirect("/kb/%d/"%e.id)



# Django settings for noc project.
# Do not modify this file directly
# Edit etc/noc.conf instead
import ConfigParser

config=ConfigParser.SafeConfigParser()
config.read("etc/noc.defaults")
config.read("etc/noc.conf")

DEBUG = config.get("main","debug")
TEMPLATE_DEBUG = DEBUG

ADMINS=[]
for a in config.get("main","admin_emails").split(","):
    a=a.strip()
    if not a:
        continue
    n,d=a.split("@")
    ADMINS.append((n,a))

MANAGERS = ADMINS

SERVER_EMAIL      = config.get("main","server_email")

DATABASE_ENGINE   = config.get("database","engine")
DATABASE_NAME     = config.get("database","name")
DATABASE_USER     = config.get("database","user")
DATABASE_PASSWORD = config.get("database","password")
DATABASE_HOST     = config.get("database","host")
DATABASE_PORT     = config.get("database","port")

TIME_ZONE = config.get("main","timezone")
LANGUAGE_CODE = config.get("main","language_code")
# Set up date and time formats
DATE_FORMAT     = config.get("main","date_format")
TIME_FORMAT     = config.get("main","time_format")
DATETIME_FORMAT = config.get("main","datetime_format")

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT.
# Example: "http://media.lawrence.com"
MEDIA_URL = ''

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = config.get("main","secret_key")

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)
# 
TEMPLATE_CONTEXT_PROCESSORS= (
    'django.core.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'noc.lib.render.setup_processor',
    )
#
MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
)

ROOT_URLCONF = 'noc.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    ".",
    "template"
)

CACHE_BACKEND = 'locmem:///'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    "south",
    
    "noc.main",
    "noc.sa",
    "noc.fm",
    "noc.cm",
    "noc.ip",
    "noc.vc",
    "noc.dns",
    "noc.peer",
    "noc.kb",
)

FORCE_SCRIPT_NAME=""


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Models for KB application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from django.db import models
from django.db.models import Q
from noc.main.models import Language,database_storage
from noc.main.menu import Menu
from django.contrib.auth.models import User
from noc.kb.parsers import parser_registry
from noc.main.search import SearchResult
from noc.lib.validators import is_int
import difflib

##
## Register all wiki-syntax parsers
##
parser_registry.register_all()
##
## KB Categories
##
class KBCategory(models.Model):
    class Meta:
        verbose_name="KB Category"
        verbose_name_plural="KB Categories"
        ordering=("name",)
    name=models.CharField("Name",max_length=64,unique=True)
    def __unicode__(self):
        return self.name
##
## KB Entry
##
class KBEntry(models.Model):
    class Meta:
        verbose_name="KB Entry"
        verbose_name_plural="KB Entries"
        ordering=("id",)
    subject=models.CharField("Subject",max_length=256)
    body=models.TextField("Body")
    language=models.ForeignKey(Language,verbose_name="Language",limit_choices_to={"is_active":True})
    markup_language=models.CharField("Markup Language",max_length="16",choices=parser_registry.choices)
    categories=models.ManyToManyField(KBCategory,verbose_name="Categories",null=True,blank=True)
    def __unicode__(self):
        return u"KB%d: %s"%(self.id,self.subject)
    ## Wiki parser class
    def _parser(self):
        return parser_registry[self.markup_language]
    parser=property(_parser)
    ## Returns parsed HTML
    def _html(self):
        return self.parser.to_html(self)
    html=property(_html)
    #
    def view_link(self):
        return "<A HREF='/kb/%d/'>View</A>"%self.id
    view_link.short_description="View"
    view_link.allow_tags=True
    ##
    ## Search engine
    ##
    @classmethod
    def search(cls,user,query,limit):
        if user.has_perm("kb.change_kbentry"):
            q=Q(subject__icontains=query)|Q(body__icontains=query)
            if is_int(query):
                q|=Q(id=int(query))
            elif query.startswith("KB") and is_int(query[2:]):
                q|=Q(id=int(query[2:]))
            for r in KBEntry.objects.filter(q):
                try:
                    idx=r.body.index(query)
                    text=r.body[idx-100:idx+len(query)+100]
                except ValueError:
                    text=r.body[:100]
                yield SearchResult(url="/kb/%d/"%r.id,
                    title="KB%d: %s"%(r.id,r.subject),
                    text=text,
                    relevancy=1.0)
    ##
    ## Returns latest KBEntryHistory record
    ##
    def _last_history(self):
        return self.kbentryhistory_set.order_by("-timestamp")[0]
    last_history=property(_last_history)
    ##
    ## Returns a list of last modified KB Entries
    ##
    @classmethod
    def last_modified(self,num=20):
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT kb_entry_id,MAX(timestamp) FROM kb_kbentryhistory GROUP BY 1 ORDER BY 2 DESC LIMIT %d"%num)
        return [KBEntry.objects.get(id=r[0]) for r in c.fetchall()]
    ##
    ## Write article preview log
    ##
    def log_preview(self,user):
        KBEntryPreviewLog(kb_entry=self,user=user).save()
    ##
    ## Returns preview count
    ##
    def _preview_count(self):
        return self.kbentrypreviewlog_set.count()
    preview_count=property(_preview_count)
    ##
    ## Returns most popular articles
    ##
    @classmethod
    def most_popular(self,num=20):
        from django.db import connection
        c=connection.cursor()
        c.execute("SELECT kb_entry_id,COUNT(*) FROM kb_kbentrypreviewlog GROUP BY 1 ORDER BY 2 DESC LIMIT %d"%num)
        return [KBEntry.objects.get(id=r[0]) for r in c.fetchall()]
    ##
    ## Callable for KBEntryAttachment.file.upload_to
    ##
    @classmethod
    def upload_to(cls,instance,filename):
        return "/kb/%d/%s"%(instance.kb_entry.id,filename)
    ##
    ## Returns a list of visible attachments
    ##
    def _visible_attachments(self):
        return [{"name":x.name,"size":x.size,"mtime":x.mtime,"url":x.url,"description":x.description}
            for x in self.kbentryattachment_set.filter(is_hidden=False).order_by("name")]
    visible_attachments=property(_visible_attachments)
    ##
    ##
    ##
    def _has_visible_attachments(self):
        return self.kbentryattachment_set.filter(is_hidden=False).count()>0
    has_visible_attachments=property(_has_visible_attachments)
    ##
    ## save model, compute body's diff and save event history
    ##
    def save(self,user=None,timestamp=None):
        if self.id:
            old_body=KBEntry.objects.get(id=self.id).body
        else:
            old_body=""
        super(KBEntry,self).save()
        if old_body!=self.body:
            diff="\n".join(difflib.unified_diff(self.body.splitlines(),old_body.splitlines()))
            KBEntryHistory(kb_entry=self,user=user,diff=diff,timestamp=timestamp).save()
    ##
    ## Check has KBEntry any bookmarks
    ##
    def is_bookmarked(self,user=None):
        # Check Global bookmarks
        if KBGlobalBookmark.objects.filter(kb_entry=self).count()>0:
            return True
        if user and KBUserBookmark.objects.filter(kb_entry=self,user=user).count()>0:
            return True
        return False
    ##
    ## Set user bookmark
    ##
    def set_user_bookmark(self,user):
        if KBUserBookmark.objects.filter(kb_entry=self,user=user).count()==0:
            KBUserBookmark(kb_entry=self,user=user).save()
    ##
    ## Uset user bookmark
    ##
    def unset_user_bookmark(self,user):
        for b in  KBUserBookmark.objects.filter(kb_entry=self,user=user):
            b.delete()
##
## Attachments
##
class KBEntryAttachment(models.Model):
    class Meta:
        verbose_name="KB Entry Attachment"
        verbose_name_plural="KB Entry Attachments"
        unique_together=[("kb_entry","name")]
    kb_entry=models.ForeignKey(KBEntry,verbose_name="KB Entry")
    name=models.CharField("Name",max_length=256)
    description=models.CharField("Description",max_length=256,null=True,blank=True)
    is_hidden=models.BooleanField("Is Hidden",default=False)
    file=models.FileField("File",upload_to=KBEntry.upload_to,storage=database_storage)
    def __unicode__(self):
        return u"%d: %s"%(self.kb_entry.id,self.name)
    ##
    ## Delete object on database storage too
    ##
    def delete(self):
        super(KBEntryAttachment,self).delete()
        self.file.storage.delete(self.file.name)
    ##
    ## File size
    ##
    def _size(self):
        s=self.file.storage.stat(self.file.name)
        if s:
            return s["size"]
        else:
            return None
    size=property(_size)
    ##
    ## File mtime
    ##
    def _mtime(self):
        s=self.file.storage.stat(self.file.name)
        if s:
            return s["mtime"]
        else:
            return None
    mtime=property(_mtime)
    ##
    ## Download URL
    ##
    def _url(self):
        return "/kb/%d/attachment/%s/"%(self.kb_entry.id,self.name)
    url=property(_url)
    ##
    ## Search engine
    ##
    @classmethod
    def search(cls,user,query,limit):
        if user.has_perm("kb.change_kbentry"):
            q=Q(name__icontains=query)|Q(description__icontains=query)
            for r in KBEntryAttachment.objects.filter(q):
                yield SearchResult(url="/kb/%d/"%r.kb_entry.id,
                    title="KB%d: %s"%(r.kb_entry.id,r.kb_entry.subject),
                    text="Attachement: %s (%s)"%(r.name,r.description),
                    relevancy=1.0)

##
## Modification History
##
class KBEntryHistory(models.Model):
    class Meta:
        verbose_name="KB Entry History"
        verbose_name_plural="KB Entry Histories"
        ordering=("kb_entry","timestamp")
    kb_entry=models.ForeignKey(KBEntry,verbose_name="KB Entry")
    timestamp=models.DateTimeField("Timestamp",auto_now_add=True)
    user=models.ForeignKey(User,verbose_name="User")
    diff=models.TextField("Diff")
##
## Preview Log
##
class KBEntryPreviewLog(models.Model):
    class Meta:
        verbose_name="KB Entry Preview Log"
        verbose_name_plural="KB Entry Preview Log"
    kb_entry=models.ForeignKey(KBEntry,verbose_name="KB Entry")
    timestamp=models.DateTimeField("Timestamp",auto_now_add=True)
    user=models.ForeignKey(User,verbose_name="User")
##
## Global Bookmarks
##
class KBGlobalBookmark(models.Model):
    class Meta:
        verbose_name="KB Global Bookmark"
        verbose_name_plural="KB Global Bookmarks"
    kb_entry=models.ForeignKey(KBEntry,verbose_name="KBEntry",unique=True)
    def __unicode__(self):
        return unicode(self.kb_entry)
##
## User Bookmarks
##
class KBUserBookmark(models.Model):
    class Meta:
        verbose_name="KB User Bookmark"
        verbose_name_plural="KB User Bookmarks"
        unique_together=[("user","kb_entry")]
    user=models.ForeignKey(User,verbose_name="User")
    kb_entry=models.ForeignKey(KBEntry,verbose_name="KBEntry")
    def __unicode__(self):
        return u"%s: %s"%(unicode(self.user),unicode(self.kb_entry))
##
## Application menu
##
class AppMenu(Menu):
    app="kb"
    title="Knowledge Base"
    items=[
        ("Knowledge Base", "/kb/", "kb.change_kbentry"),
        ("Setup",[
            ("Categories","/admin/kb/kbcategory/", "kb.change_kbcategory"),
            ("Entries",   "/admin/kb/kbentry/",    "kb.change_kbentry"),
            ("Global Bookmarks", "/admin/kb/kbglobalbookmark/", "kb.change_kbglobalbookmark"),
            ("User Bookmarks",  "/admin/kb/kbuserbookmark/",    "kb.change_kbuserbookmark"),
        ])
    ]




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django's standard views module
## For VC application
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django import forms
from django.contrib.auth.decorators import permission_required
from noc.lib.render import render,render_success
from noc.vc.models import VCDomain,VC
from noc.sa.models import ManagedObject, profile_registry
from noc.settings import config
from xmlrpclib import ServerProxy, Error

##
## Profiles having get_vlans script
##
get_vlans_profiles=[p.name for p in profile_registry.classes.values() if "get_vlans" in p.scripts]
##
## Import vlans from switches via SA
##
class SAImportVLANsForm(forms.Form):
    vc_domain=forms.ModelChoiceField(label="VC Domain",queryset=VCDomain.objects)
    managed_object=forms.ModelChoiceField(label="Managed Object",queryset=ManagedObject.objects.filter(profile_name__in=get_vlans_profiles))

@permission_required("vc.add_vc")
def sa_import_vlans(request):
    def update_vlans(vc_domain,mo):
        script="%s.get_vlans"%mo.profile_name
        count=0
        server=ServerProxy("http://%s:%d"%(config.get("xmlrpc","server"),config.getint("xmlrpc","port")))
        result=server.script(script,mo.id,{})
        for v in result:
            vlan_id=v["vlan_id"]
            name=v.get("name","VLAN%d"%vlan_id)
            try:
                vc=VC.objects.get(vc_domain=vc_domain,type="q",l1=vlan_id)
            except VC.DoesNotExist:
                vc=VC(vc_domain=vc_domain,type="q",l1=vlan_id,l2=0,description=name)
                vc.save()
                count+=1
        return count
    if request.POST:
        form=SAImportVLANsForm(request.POST)
        if form.is_valid():
            count=update_vlans(form.cleaned_data["vc_domain"],form.cleaned_data["managed_object"])
            return render_success(request,"VLANs are imported","%d VLANs are imported"%count)
    else:
        form=SAImportVLANsForm()
    return render(request,"vc/sa_import_vlans.html",{"form":form})



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

# Values:
#   (QueryType,Command,ArgRequired)
DEFAULT={
    "Cisco.IOS": [
        ("ipv4:bgp",          "show ip bgp %(query)s"),
        ("advertised-routes", "show ip bgp neighbors %(query)s advertised-routes"),
        ("summary",           "show ip bgp summary"),
        ("ping",              "ping %(query)s"),
        ("trace",             "traceroute %(query)s"),
    ],
    "Juniper.JUNOS": [
        ("ipv4:bgp",           "show route table inet.0 %(query)s detail"),
        ("advertised-routes" , "show route advertising-protocol bgp %(query)s %(query)s"),
        ("summary",            "show bgp summary"),
        ("ping",               "ping count 5 %(query)s"),
        ("trace",              "traceroute %(query)s as-number-lookup"),
    ]
}

class Migration:
    def forwards(self):
        qtype={}
        for ppt in DEFAULT:
            if db.execute("SELECT COUNT(*) FROM peer_peeringpointtype WHERE name=%s",[ppt])[0][0]==0:
                db.execute("INSERT INTO peer_peeringpointtype(name) VALUES(%s)",[ppt])
            ppt_id=db.execute("SELECT id FROM peer_peeringpointtype WHERE name=%s",[ppt])[0][0]
            for k,v in DEFAULT[ppt]:
                if not k in qtype:
                    db.execute("INSERT INTO peer_lgquerytype(name) VALUES(%s)",[k])
                    qtype[k]=db.execute("SELECT id FROM peer_lgquerytype WHERE name=%s",[k])[0][0]
                q=qtype[k]
                if db.execute("SELECT COUNT(*) FROM peer_lgquerycommand WHERE peering_point_type_id=%s AND query_type_id=%s",[ppt_id,q])[0][0]==0:
                    db.execute("INSERT INTO peer_lgquerycommand(peering_point_type_id,query_type_id,command) VALUES(%s,%s,%s)",[ppt_id,q,v])
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetConfig

class Script(noc.sa.script.Script):
    name="Audiocodes.Mediant2000.get_config"
    implements=[IGetConfig]
    def execute(self):
        if self.access_profile.scheme in [self.TELNET,self.SSH]:
            self.cli("conf")
            config=self.cli("cf get")
        elif self.access_profile.scheme==self.HTTP:
            config=self.http.get("/FS/BOARD.ini")
        else:
            raise Exception("Unsupported access scheme")
        return self.cleaned_config(config)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Vendor: Huawei
## OS:     UMG8900 media gateway
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.profiles
from noc.sa.protocols.sae_pb2 import TELNET

class Profile(noc.sa.profiles.Profile):
    name="Huawei.UMG8900"
    supported_schemes=[TELNET]
    pattern_username="Login :"
    pattern_password="Password :"
    pattern_more=r"^Press CTRL\+C to break, other key to continue\.\.\."
    pattern_prompt=r"mml>"
    command_more=" "
    rogue_chars=["\r"]
    config_volatile=[r"^\+\+\+.*?$"]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## now macro
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from noc.kb.parsers.macros import Macro as MacroBase
from django.utils.dateformat import DateFormat
import datetime
##
## now macro
##     Returns current date and time
##     Optional arguments
##     format - PHP date()-like format string
##
##
class Macro(MacroBase):
    name="now"
    @classmethod
    def handle(cls,args,text):
        if "format" in args:
            format_string=args["format"]
        else:
            format_string="Y.m.d H:i:s"
        df=DateFormat(datetime.datetime.now())
        return df.format(format_string)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Simple RIPE whois client
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import logging
from noc.lib.nbsocket import ConnectedTCPSocket

WHOIS_SERVER="whois.ripe.net"
WHOIS_PORT=43

class Whois(ConnectedTCPSocket):
    def __init__(self,factory,query,callback=None,fields=None):
        ConnectedTCPSocket.__init__(self,factory,WHOIS_SERVER,WHOIS_PORT)
        self.query=query.strip()
        self.output=[]
        self.callback=callback
        self.fields=fields
        logging.debug("whois(%s)"%self.query)
        
    def on_connect(self):
        self.write(self.query+"\r\n")
    
    def on_read(self,data):
        self.output+=[data]
    
    def on_close(self):
        if self.callback:
            self.callback(self.format())
            
    def format(self):
        out=[]
        last=None
        for l in "".join(self.output).split("\n"):
            l=l.strip()
            if not l or l.startswith("%"):
                continue
            L=l.split(":")
            if len(L)!=2:
                L=[last,L[0]]
            else:
                last=L[0]
            if self.fields is None or L[0] in self.fields:
                out.append([L[0],L[1].strip()])
        return out
##
## Simple whois client function for testing
#3
def whois(q,fields=None):
    from noc.lib.nbsocket import SocketFactory
    f=SocketFactory()
    s=[]
    w=Whois(f,q,s.append,fields)
    f.run()
    return s



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetMACAddressTable
import re

rx_line=re.compile(r"^\*\s+(?P<vlan_id>\d+)\s+(?P<mac>\S+)\s+(?P<type>\S+)\s+\S+\s+\S+\s+(?P<interfaces>.*)$")

class Script(noc.sa.script.Script):
    name="Cisco.IOS.get_mac_address_table"
    implements=[IGetMACAddressTable]
    def execute(self,interface=None,vlan=None,mac=None):
        cmd="show mac-address-table"
        if mac is not None:
            cmd+=" address %s"%self.profile.convert_mac(mac)
        if interface is not None:
            cmd+=" interface %s"%interface
        if vlan is not None:
            cmd+=" vlan %s"%vlan
        self.cli("terminal length 0")
        vlans=self.cli(cmd)
        r=[]
        for l in vlans.split("\n"):
            match=rx_line.match(l.strip())
            if match:
                r.append({
                    "vlan_id"   : match.group("vlan_id"),
                    "mac"       : match.group("mac"),
                    "interfaces": [match.group("interfaces")],
                    "type"      : {"dynamic":"D","static":"S"}[match.group("type")],
                })
        return r




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        db.execute("DROP FUNCTION update_event_classification(INTEGER,INTEGER,INTEGER,INTEGER,TEXT,TEXT,TEXT[][])")
        db.execute(PROC)
    
    def backwards(self):
        pass

PROC="""
CREATE OR REPLACE
FUNCTION update_event_classification(INTEGER,INTEGER,INTEGER,INTEGER,INTEGER,TEXT,TEXT,TEXT[][])
RETURNS VOID
AS
$$
DECLARE
    p_event_id          ALIAS FOR $1;
    p_rule_id           ALIAS FOR $2;
    p_event_class_id    ALIAS FOR $3;
    p_event_category_id ALIAS FOR $4; 
    p_event_priority_id ALIAS FOR $5;
    p_subject           ALIAS FOR $6;
    p_body              ALIAS FOR $7;
    p_vars              ALIAS FOR $8;
    rule_name           TEXT;
BEGIN
    -- Delete calculated event data
    DELETE FROM fm_eventdata
    WHERE event_id=p_event_id
        AND "type" IN ('R','V');
    -- Fill new event data
    FOR i IN array_lower(p_vars,1) .. array_upper(p_vars,1) LOOP
        INSERT INTO fm_eventdata(event_id,"type",key,value)
        VALUES(p_event_id,p_vars[i][1],p_vars[i][2],p_vars[i][3]);
    END LOOP;
    -- Update event itself
    UPDATE fm_event
    SET
        event_class_id=p_event_class_id,
        event_category_id=p_event_category_id,
        event_priority_id=p_event_priority_id,
        status='A',
        subject=p_subject,
        body=p_body
    WHERE id=p_event_id;
    -- Find rule name
    IF p_rule_id=0 THEN
        rule_name:='DEFAULT';
    ELSE
        SELECT name
        INTO   rule_name
        FROM   fm_eventclassificationrule
        WHERE  id=p_rule_id;
    END IF;
    -- Write event log
    INSERT INTO fm_eventlog(event_id,timestamp,from_status,to_status,message)
    VALUES(p_event_id,'now','U','A','CLASSIFICATION RULE: '||rule_name);
END;
$$ LANGUAGE plpgsql;
"""


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
import noc.main.report
from noc.dns.generators import generator_registry

class Report(noc.main.report.Report):
    name="dns.supported_servers"
    title="Supported nameservers"
    requires_cursor=False
    columns=[Column("Type")]
    
    def get_queryset(self):
        r=[x[0] for x in generator_registry.choices]
        r.sort()
        return [(x,) for x in r]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## admin.py
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.contrib import admin
from noc.ip.models import VRFGroup,VRF,IPv4BlockAccess,IPv4Block,IPv4Address
from noc.peer.models import AS
##
## Admin for VRFGroup
##
class VRFGroupAdmin(admin.ModelAdmin):
    list_display=["name","unique_addresses"]
    search_fields=["name"]
##
## Admin or VRF
##
class VRFAdmin(admin.ModelAdmin):
    list_display=["rd","name","vrf_group"]
    search_fields=["name","rd"]
    list_filter=["vrf_group"]
    def save_model(self, request, obj, form, change):
        vrf=form.save()
        # Create 0.0.0.0/0 prefix if not exists
        try:
            IPv4Block.objects.get(vrf=vrf,prefix="0.0.0.0/0")
        except IPv4Block.DoesNotExist:
            IPv4Block(vrf=vrf,prefix="0.0.0.0/0",description="root",asn=AS.default_as(),modified_by=request.user).save()
##
## Admin for IPv4BlockAccess
##
class IPv4BlockAccessAdmin(admin.ModelAdmin):
    list_display=["user","vrf","prefix"]
    list_filter=["user","vrf"]
    search_fields=["user","prefix"]
##
## Admin for IPv4Block
##
class IPv4BlockAdmin(admin.ModelAdmin):
    list_display=["prefix","vrf","description"]
    list_filter=["vrf","asn"]
    search_fields=["prefix","description"]
##
## Admin for IPv4Address
##
class IPv4AddressAdmin(admin.ModelAdmin):
    list_display=["ip","fqdn","vrf","description"]
    list_filter=["vrf"]
    search_fields=["ip","fqdn","description"]

admin.site.register(VRFGroup,VRFGroupAdmin)
admin.site.register(VRF,VRFAdmin)
admin.site.register(IPv4BlockAccess,IPv4BlockAccessAdmin)
admin.site.register(IPv4Block,IPv4BlockAdmin)
admin.site.register(IPv4Address,IPv4AddressAdmin)



#!/usr/bin/env python
#
# Service Activation Engine
#
import os,sys,getopt

if __name__ == '__main__':
    d=os.path.dirname(sys.argv[0])
    sys.path.insert(0,os.path.join(d,"..",".."))
    sys.path.insert(0,os.path.join(d,".."))
    sys.path.insert(0,d)
    from noc.sa.activator import Activator
    activator=Activator()
    activator.process_command()


# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.peer.models import *

class Migration:
    
    def forwards(self):
        rir_id=db.execute("SELECT id FROM peer_rir LIMIT 1")[0][0]
        lir_to_rir={}
        for lir_id,name in db.execute("SELECT id,name FROM peer_lir"):
            db.execute("INSERT INTO peer_maintainer(maintainer,description,auth,rir_id) VALUES(%s,%s,%s,%s)",
                [name,name,'NOAUTH',rir_id])
            lir_to_rir[lir_id]=db.execute("SELECT id FROM peer_maintainer WHERE maintainer=%s",[name])[0][0]
        db.add_column("peer_as","maintainer",models.ForeignKey(Maintainer,verbose_name="Maintainer",null=True,blank=True))
        for as_id,lir_id in db.execute("SELECT id,lir_id FROM peer_as"):
            db.execute("UPDATE peer_as SET maintainer_id=%s WHERE id=%s",[lir_to_rir[lir_id],as_id])
        db.execute("ALTER TABLE peer_as ALTER maintainer_id SET NOT NULL")
        db.delete_column("peer_as","lir_id")
        db.delete_table("peer_lir")
    
    def backwards(self):
        "Write your backwards migration here"



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.script
from noc.sa.interfaces import IGetConfig

class Script(noc.sa.script.Script):
    name="Juniper.JUNOS.get_config"
    implements=[IGetConfig]
    def execute(self):
        self.cli("set cli screen-length 0")
        config=self.cli("show configuration")
        return self.cleaned_config(config)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## .. supported:: Profile
## extension
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from __future__ import with_statement
from sphinx.util.compat import Directive
from docutils import nodes
import csv,sets

class supported(nodes.Element): pass

class Supported(Directive):
    has_content=True
    required_arguments=1
    def run(self):
        node=supported()
        node.document=self.state.document
        node.line=self.lineno
        node["profile"]=self.arguments[0]
        return [node]

def html_visit_supported(self,node):
    body=[]
    reader=csv.reader(open("../../../../local/supported.csv"))
    profile=node["profile"]
    se={}
    for r in [l for l in reader if l[0]==profile]:
        profile,model,version=r
        if model not in se:
            se[model]=sets.Set()
        se[model].add(version)
    body+=["<table border=\"1\" class=\"docutils\">"]
    for m in sorted(se.keys()):
        body+=["<tr><td>%s</td><td>%s</td></tr>"%(m,", ".join(sorted(se[m])))]
    body+=["</table>"]
    self.body.append("\n".join(body))
    raise nodes.SkipNode

def setup(app):
    app.add_node(supported,
        html=(html_visit_supported,None)
        )
    app.add_directive("supported",Supported)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Cisco IOS 802.11 classification rules
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classification import ClassificationRule,Expression
from noc.fm.rules.classes.dot11 import *

##
## CISCO.IOS dot11 Associated
##
class CISCO_IOS_dot11_Associated_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS dot11 Associated SNMP"
    event_class=Dot11Associated
    preference=1000
    patterns=[
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%DOT11-6-ASSOC:.+?(?P<raw_mac>\S+) (?:A|Rea)ssociated"),
        (r"^profile$",r"^Cisco\.IOS$"),
        Expression("mac","decode_mac(raw_mac)")
    ]
##
## Cisco.IOS dot11 Disassociated
##
class Cisco_IOS_dot11_Disassociated_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS dot11 Disassociated SNMP"
    event_class=Dot11Disassociated
    preference=1000
    patterns=[
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%DOT11-6-DISASSOC: .+?(?P<raw_mac>\S+) Reason"),
        (r"^profile$",r"^Cisco\.IOS$"),
        Expression("mac","decode_mac(raw_mac)")
    ]
##
## Cisco.IOS dot11 Max Retries SYSLOG
##
class Cisco_IOS_dot11_Max_Retries_SYSLOG_Rule(ClassificationRule):
    name="Cisco.IOS dot11 Max Retries SYSLOG"
    event_class=Dot11MaxRetries
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^syslog$"),
        (r"^message$",r"%DOT11-4-MAXRETRIES: Packet to client (?P<raw_mac>\S+) reached max retries, removing the client$"),
        Expression("mac","decode_mac(raw_mac)")
    ]
##
## Cisco.IOS dot11 Max Retries SYSLOG SNMP
##
class Cisco_IOS_dot11_Max_Retries_SYSLOG_SNMP_Rule(ClassificationRule):
    name="Cisco.IOS dot11 Max Retries SYSLOG SNMP"
    event_class=Dot11MaxRetries
    preference=1000
    patterns=[
        (r"^profile$",r"^Cisco\.IOS$"),
        (r"^source$",r"^SNMP Trap$"),
        (r"^1\.3\.6\.1\.6\.3\.1\.1\.4\.1\.0$",r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.2\.0\.1$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.5\.",r"^Packet to client (?P<raw_mac>\S+) reached max retries, removing the client$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.2\.",r"^DOT11$"),
        (r"^1\.3\.6\.1\.4\.1\.9\.9\.41\.1\.2\.3\.1\.4\.",r"^MAXRETRIES$"),
        Expression("mac","decode_mac(raw_mac)")
    ]



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Power-over-Ethernet events
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.fm.rules.classes import EventClass,Var
##
## PoE Power Granted
##
class PoEPowerGranted(EventClass):
    name="PoE Power Granted"
    category="NETWORK"
    priority="NORMAL"
    subject_template="PoE Power Granted on {{interface}}"
    body_template="""PoE Power Granted on interface {{interface}}"""
    class Vars:
        interface=Var(required=True)
##
## PoE Power Disconnect
##
class PoEPowerDisconnect(EventClass):
    name="PoE Power Disconnect"
    category="NETWORK"
    priority="NORMAL"
    subject_template="PoE Power Disconnected on {{interface}}"
    body_template="""PoE Power Disconnected on interface {{interface}}"""
    class Vars:
        interface=Var(required=True)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Form widgets
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django import forms
from django.http import HttpResponse

##
## Autocomplete widget
##
class AutoCompleteTextInput(forms.TextInput):
    def __init__(self,lookup_url,*args,**kwargs):
        super(AutoCompleteTextInput,self).__init__(*args,**kwargs)
        self.lookup_url=lookup_url
    def render(self,name,value=None,attrs=None):
        return "%s<script type=\"text/javascript\">$(\"#%s\").autocomplete(\"%s\",{minChars:3,mustMatch:1});</script>"%(
            super(AutoCompleteTextInput,self).render(name,value,attrs),attrs["id"],self.lookup_url
        )
##
## Autocomplete lookup function:
## 
def lookup(request,func):
    result=[]
    if request.GET and "q" in request.GET:
        q=request.GET["q"]
        if len(q)>2: # Ignore requests shorter than 3 letters
            result=list(func(q))
    return HttpResponse("\n".join(result), mimetype='text/plain')



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.periodic
from django.db.models import Q
import datetime,logging

class Task(noc.sa.periodic.Task):
    name="cm.config_pull"
    description=""
    
    def execute(self):
        from noc.cm.models import Config
        for o in Config.objects.filter((Q(next_pull__lt=datetime.datetime.now())|Q(next_pull__isnull=True))&Q(pull_every__isnull=False)).order_by("next_pull"):
            logging.debug("Pulling %s",str(o))
            o.pull(self.sae)
        return True




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Abstract Wiki parsers class
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from noc.lib.registry import Registry
from noc.lib.validators import is_int
from noc.settings import config
from noc.kb.parsers.macros import macro_registry

macro_registry.register_all()

##
## Parser Registry
##
class ParserRegistry(Registry):
    name="ParserRegistry"
    subdir="parsers"
    classname="Parser"
    apps=["noc.kb"]
parser_registry=ParserRegistry()

##
## Metaclass for Parser
##
class ParserBase(type):
    def __new__(cls,name,bases,attrs):
        m=type.__new__(cls,name,bases,attrs)
        parser_registry.register(m.name,m)
        return m
##
## Abstract parser class
##
class Parser(object):
    __metaclass__=ParserBase
    name=None
    ##
    ## Convert wiki syntax to HTML
    ##
    @classmethod
    def to_html(cls,kb_entry):
        raise "Does not implemented"
    ##
    ## Check syntax
    ##
    @classmethod
    def check(cls,text):
        cls.to_html(text)
    ##
    ## Convert extracted link to URL
    ## Following link types are supported:
    ##   KB<n> - Link to Knowledge Base article <n>
    ##   TT<n> - Link to Trouble Ticket <n>
    ##   attach:<name> - Link to attachment <name>
    @classmethod
    def convert_link(cls,kb_entry,link,text=None):
        if text is None:
            text=link
        if link.startswith("KB") and is_int(link[2:]):
            return u"<a href='/kb/%s/'>%s</a>"%(link[2:],text)
        elif link.startswith("TT"):
            tt={"tt":link[2:]}
            tt_url=config.get("tt","url",tt)%tt
            return u"<a href='%s'>%s</a>"%(tt_url,text)
        elif link.startswith("attach:"):
            if text==link:
                text=link[7:]
            link=link[7:]
            return u"<a href='/kb/%d/attachment/%s/'>%s</a>"%(kb_entry.id,link,text)
        elif link.startswith("attachment:"):
            if text==link:
                text=link[11:]
            link=link[11:]
            return u"<a href='/kb/%d/attachment/%s/'>%s</a>"%(kb_entry.id,link,text)
        else:
            return u"a<href='%s'>%s</a>"%(link,text)
    ##
    ## Convert attachment ref
    ##
    @classmethod
    def convert_attach(cls,kb_entry,href):
        if href.startswith("http"):
            return href
        elif href.startswith("attach:"):
            name=href[7:]
        elif href.startswith("attachment:"):
            href=href[11:]
        return "/kb/%d/attachment/%s/"%(kb_entry.id,href)




# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Vendor: Juniper
## OS:     JUNOS
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
import noc.sa.profiles
from noc.sa.protocols.sae_pb2 import TELNET,SSH

class Profile(noc.sa.profiles.Profile):
    name="Juniper.JUNOS"
    supported_schemes=[TELNET,SSH]
    pattern_prompt="^({master}\n)?\S*>"
    pattern_more=r"^---\(more.*?\)---"
    command_more=" "
    pattern_lg_as_path_list=r"(?<=AS path: )(\d+(?: \d+)*)"
    pattern_lg_best_path=r"^(\s+[+*].+?\s+Router ID: \S+)"
    oid_trap_config_changed="1.3.6.1.4.1.2636.4.5.0.1" # jnxCmCfgChange
    
    def generate_prefix_list(self,name,pl,strict=True):
        if strict:
            p="        route-filter %s exact;"
        else:
            p="        route-filter %s orlonger;"
        out=["term pass {","    from {"]+[p%x for x in pl]+["    }","    then next policy;"]
        out+=["term reject {","    then reject;","}"]
        return "\n".join(out)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.main.report import Column
from noc.settings import config
import noc.main.report
import os,stat,datetime

class Report(noc.main.report.Report):
    name="main.backups"
    title="Backups Status"
    requires_cursor=False
    columns=[
        Column("File"),
        Column("Date Created"),
        Column("Size (bytes)",align="RIGHT")
    ]
    def get_queryset(self):
        bd=config.get("path","backup_dir")
        if not os.path.isdir(bd):
            return []
        r=[]
        for f in [f for f in os.listdir(bd) if f.startswith("noc-") and (f.endswith(".dump") or f.endswith(".tar.gz"))]:
            s=os.stat(os.path.join(bd,f))
            r.append([f,datetime.datetime.fromtimestamp(s[stat.ST_MTIME]),s[stat.ST_SIZE]])
        return sorted(r,lambda x,y:cmp(x[1],y[1]))




from south.db import db
from noc.kb.models import *

class Migration:
    
    def forwards(self):
        
        
        # Mock Models
        KBEntry = db.mock_model(model_name='KBEntry', db_table='kb_kbentry', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        User = db.mock_model(model_name='User', db_table='auth_user', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        
        # Model 'KBEntryPreviewLog'
        db.create_table('kb_kbentrypreviewlog', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('kb_entry', models.ForeignKey(KBEntry,verbose_name="KB Entry")),
            ('timestamp', models.DateTimeField("Timestamp",auto_now_add=True)),
            ('user', models.ForeignKey(User,verbose_name=User))
        ))
        
        db.send_create_signal('kb', ['KBEntryPreviewLog'])
    
    def backwards(self):
        db.delete_table('kb_kbentrypreviewlog')
        



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from south.db import db
from noc.dns.models import *

class Migration:
    def forwards(self):
        db.add_column("dns_dnsserver","generator_name",models.CharField("Generator",max_length=32,default="BINDv9"))
        db.execute("UPDATE dns_dnsserver SET generator_name=%s",["BINDv9"])
    
    def backwards(self):
        raise "No backwards way"




from south.db import db
from noc.fm.models import *

class Migration:
    
    def forwards(self):
        Event = db.mock_model(model_name='Event', db_table='fm_event', db_tablespace='', pk_field_name='id', pk_field_type=models.AutoField)
        db.add_column("fm_event","status",models.CharField("Status",max_length=1,default="U"))
        db.add_column("fm_event","active_till",models.DateTimeField("Active Till",blank=True,null=True))
        db.add_column("fm_event","close_timestamp",models.DateTimeField("Close Timestamp",blank=True,null=True))
        db.add_column("fm_event","root",models.ForeignKey(Event,blank=True,null=True))
    
    def backwards(self):
        db.delete_column("fm_event","status")
        db.delete_column("fm_event","active_till")
        db.delete_column("fm_event","close_timestamp")
        dv.delete_column("fm_event","root")



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Django URL dispatcher for module SA
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from django.conf.urls.defaults import *
from django.contrib.auth.decorators import login_required
from noc.sa.views import object_scripts,object_script,tools,upload_managed_objects

urlpatterns = patterns ( "",
    (r"^(?P<object_id>\d+)/scripts/$",  login_required(object_scripts)),
    (r"^(?P<object_id>\d+)/scripts/(?P<script>[^/]+)/$", login_required(object_script)),
    (r"^tools/$",                       login_required(tools)),
    (r"^tools/upload_managed_objects/", login_required(upload_managed_objects)),
)



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## HttpResponse wrappers and shortcuts
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.http import HttpResponse
from django.utils.simplejson.encoder import JSONEncoder
from noc.settings import config

##
## Setup Context Processor.
## Used via TEMPLATE_CONTEXT_PROCESSORS variable in settings.py
## Adds "setup" variable to context.
## "setup" is a hash of
##      "installation_name" 
##
def setup_processor(request):
    return {
        "setup" : {
            "installation_name" : config.get("customization","installation_name"),
            "logo_url"          : config.get("customization","logo_url"),
            "logo_width"        : config.get("customization","logo_width"),
            "logo_height"       : config.get("customization","logo_height"),
        }
    }
##
## Render template within context
##
def render(request,template,dict={}):
    return render_to_response(template,dict,context_instance=RequestContext(request))
##
## Create plain text response
##
def render_plain_text(text):
    return HttpResponse(text,mimetype="text/plain")
##
## Create serialized JSON-encoded response
##
def render_json(obj):
    return HttpResponse(JSONEncoder(ensure_ascii=False).encode(obj),mimetype="text/json")
##
## Render "success" page
##
def render_success(request,subject=None,text=None):
    return render(request,"main/success.html",{"subject":subject,"text":text})
##
## Render "failure" page
##
def render_failure(request,subject=None,text=None):
    return render(request,"main/failure.html",{"subject":subject,"text":text})



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## SAE RPC Protocol
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from noc.sa.protocols.sae_pb2 import Message,Error
import struct,logging,random,time,sha,zlib
from google.protobuf.service import RpcController
from noc.sa.protocols.sae_pb2 import *
from noc.lib.nbsocket import Protocol

##
## RPC Controller
##
class Controller(RpcController):
    def __init__(self,stream):
        RpcController.__init__(self)
        self.stream=stream
        self.transaction=None
    def Reset(self):
        pass
    def Failed(self):
        pass
    def ErrorText(self):
        pass
    def StartCancel(self):
        pass
    def SetFailed(self, reason):
        pass
    def IsCancelled(self):
        pass
    def NotifyOnCancel(self,callback):
        pass

##
## Transaction structure
##
class Transaction(object):
    def __init__(self,factory,id,method=None,callback=None):
        logging.debug("Creating transaction id=%s method=%s callback=%s"%(id,method,callback))
        self.factory=factory
        self.id=id
        self.method=method
        self.callback=callback
        
    def __del__(self):
        logging.debug("Deleting transaction %s"%self.id)
        
    def commit(self,response=None,error=None):
        if self.callback:
            self.callback(self,response,error)
        self.factory.delete_transaction(self.id)
        
    def rollback(self):
        self.factory.delete_transaction(self.id)
##
## Transaction storage
##
class TransactionFactory(object):
    def __init__(self):
        self.transactions={}
        
    def __len__(self):
        return len(self.transactions)
        
    # Generate unique id
    def __get_id(self):
        while True:
            id=random.randint(0,0x7FFFFFFF)
            if id not in self.transactions:
                return id

    def has_key(self,id):
        return id in self.transactions
        
    def __getitem__(self,id):
        return self.transactions[id]
        
    # Begins transaction, remembers callback and returns transaction id
    def begin(self,id=None,method=None,callback=None):
        if id:
            if id in self.transactions:
                raise Exception("Transaction is already exists")
        else:
            id=self.__get_id()
        t=Transaction(self,id,method,callback)
        self.transactions[id]=t
        return t
        
    def delete_transaction(self,id):
        del self.transactions[id]
##
## Stream is a <len><data><len><data>..... sequence
## Where len is an 32 bit integer in network order
##
class Int32Protocol(Protocol):
    def parse_pdu(self):
        r=[]
        while len(self.in_buffer)>=4:
            l=struct.unpack("!L",self.in_buffer[:4])[0]
            if len(self.in_buffer)>=4+l:
                r+=[self.in_buffer[4:4+l]]
                self.in_buffer=self.in_buffer[4+l:]
            else:
                break
        return r
##
##
##
class CompressedInt32Protocol(Protocol):
    def parse_pdu(self):
        r=[]
        while len(self.in_buffer)>=4:
            l=struct.unpack("!L",self.in_buffer[:4])[0]
            if len(self.in_buffer)>=4+l:
                try:
                    pdu=zlib.decompress(self.in_buffer[4:4+l])
                    r+=[pdu]
                except:
                    logging.error("Failed to decompress PDU")
                self.in_buffer=self.in_buffer[4+l:]
            else:
                break
        return r
##
##
##
class RPCSocket(object):
    protocol_class=CompressedInt32Protocol
    def __init__(self,service):
        self.service=service
        self.proxy=Proxy(self,SAEService_Stub)
        self.transactions=TransactionFactory()
        self.stat_rpc_requests=0
        self.stat_rpc_responses=0
        self.stat_rpc_errors=0
        
    def on_read(self,data):
        logging.debug("on_read: %s"%repr(data))
        msg=Message()
        msg.ParseFromString(data)
        logging.debug("rpc_handle_message:\n%s"%msg)
        if msg.error.ByteSize():
            self.stat_rpc_errors+=1
            self.rpc_handle_error(msg.id,msg.error)
        elif msg.request.ByteSize():
            self.stat_rpc_requests+=1
            self.rpc_handle_request(msg.id,msg.request)
        elif msg.response.ByteSize():
            self.stat_rpc_responses+=1
            self.rpc_handle_response(msg.id,msg.response)
            
    def rpc_handle_request(self,id,request):
        logging.debug("rpc_handle_request")
        if self.transactions.has_key(id):
            self.send_error(id,ERR_TRANSACTION_EXISTS,"Transaction %s is alreasy exists"%id)
            return
        method=self.service.GetDescriptor().FindMethodByName(request.method)
        if method:
            req=self.service.GetRequestClass(method)()
            req.ParseFromString(request.serialized_request)
            logging.debug("Request accepted:\nid: %s\n%s"%(id,str(req)))
            controller=Controller(self)
            controller.transaction=self.transactions.begin(id=id,method=request.method)
            self.service.CallMethod(method,controller,req,self.send_response)
        else:
            self.send_error(id,ERR_INVALID_METHOD,"invalid method '%s'"%request.method)
        
    def rpc_handle_response(self,id,response):
        logging.debug("rpc_handle_response:\nid: %s\n%s"%(id,str(response)))
        if not self.transactions.has_key(id):
            logging.error("Invalid transaction: %s"%id)
            return
        t=self.transactions[id]
        method=self.service.GetDescriptor().FindMethodByName(t.method)
        if method:
            res=self.service.GetResponseClass(method)()
            res.ParseFromString(response.serialized_response)
            t.commit(response=res)
        else:
            logging.error("Invalid method: %s"%t.method)
            t.rollback()
    
    def rpc_handle_error(self,id,error):
        logging.debug("rpc_handle_error:\nid: %s\n%s"%(id,str(error)))
        if not self.transactions.has_key(id):
            logging.error("Invalid transaction: %s"%id)
            return
        self.transactions[id].commit(id,error=error)
        
    # Format and write SAE RPC PDU
    def write_message(self,msg):
        s=zlib.compress(msg.SerializeToString())
        self.write(struct.pack("!L",len(s))+s)
        
    def send_request(self,id,method,request):
        logging.debug("send_request\nmethod: %s\n%s"%(method,str(request)))
        m=Message()
        m.id=id
        m.request.method=method
        m.request.serialized_request=request.SerializeToString()
        self.write_message(m)
        return id
        
    def send_response(self,controller,response=None,error=None):
        id=controller.transaction.id
        logging.debug("send_response:\nid: %d\nresponse:\n%s\nerror:\n%s"%(id,str(response),str(error)))
        if not self.transactions.has_key(id):
            raise Exception("Invalid transaction")
        m=Message()
        m.id=id
        if error:
            m.error.code=error.code
            m.error.text=error.text
        if response:
            m.response.serialized_response=response.SerializeToString()
        self.write_message(m)
        controller.transaction.commit()
        
    def send_error(self,id,code,text):
        logging.debug("send_error:\nid: %s\ncode: %s\ntext: %s"%(id,code,text))
        m=Message()
        m.id=id
        m.error.code=code
        m.error.text=text
        self.write_message(m)
        
    def call(self,method,request,callback):
        t=self.transactions.begin(method=method,callback=callback)
        self.send_request(t.id,method,request)
        return t
        
    def _stats(self):
        return [
            ("rpc.requests",  self.stat_rpc_requests),
            ("rpc.responses", self.stat_rpc_responses),
            ("rpc.errors",    self.stat_rpc_errors),
        ]
    stats=property(_stats)
##
## Proxy class for RPC interface
##
class Proxy(object):
    def __init__(self,stream,stub_class):
        self.stream=stream
        self.stub=stub_class(stream.service)
    def __getattr__(self,name):
        return lambda request,callback: self.stream.call(name,request,callback)
##
## file hash for software update services
##
def file_hash(path):
    f=open(path)
    data=f.read()
    f.close()
    s=sha.new(data)
    return s.hexdigest()
##
## Generic hash for digest authetications
##
def H(s):
    return sha.new(s).hexdigest()

##
## Generate random nonce for digest authetuication
##
def get_nonce():
    ur=random.SystemRandom()
    return H(H(str(ur.random())[2:])+str(ur.random())[2:])

##
## Compute digest for Activator authentications
##
def get_digest(name,password,nonce):
    return H(H(name+":"+password)+":"+nonce)
    



# -*- coding: utf-8 -*-
##----------------------------------------------------------------------
## Copyright (C) 2007-2009 The NOC Project
## See LICENSE for details
##----------------------------------------------------------------------
"""
"""
from __future__ import with_statement
from django.db import models
from noc.sa.models import ManagedObject
from noc.settings import config
from noc.lib.fileutils import safe_rewrite
from noc.fm.triggers import event_trigger_registry
from noc.main.menu import Menu
import imp,subprocess,tempfile,os,datetime,re

##
event_trigger_registry.register_all()
##
## Python quote helper
##
def py_q(s):
    return s.replace("\"","\\\"")

##
## Exceptions
##
class MIBRequiredException(Exception):
    def __init__(self,mib,requires_mib):
        super(MIBRequiredException,self).__init__()
        self.mib=mib
        self.requires_mib=requires_mib
    def __str__(self):
        return "%s requires %s"%(self.mib,self.requires_mib)
##
class MIBNotFoundException(Exception):
    def __init__(self,mib):
        super(MIBNotFoundException,self).__init__()
        self.mib=mib
    def __str__(self):
        return "MIB not found: %s"%self.mib


##
## Regular expressions
##
rx_module_not_found=re.compile(r"{module-not-found}.*`([^']+)'")
rx_py_id=re.compile("[^0-9a-zA-Z]+")

##
## SNMP MIB
##
class MIB(models.Model):
    class Meta:
        verbose_name="MIB"
        verbose_name_plural="MIBs"
        ordering=["name"]
    name=models.CharField("Name",max_length=64,unique=True)
    description=models.TextField("Description",blank=True,null=True)
    last_updated=models.DateTimeField("Last Updated") # Latest revision of MIB
    uploaded=models.DateTimeField("Uploaded")
    
    def __unicode__(self):
        return self.name
    ##
    ## Load MIB into database
    ## smidump from libsmi used to parse MIB
    ##
    @classmethod
    def load(self,path):
        # Build SMIPATH variable for smidump to exclude locally installed MIBs
        smipath=["share/mibs","local/share/mibs"]
        # Pass MIB through smilint to detect missed modules
        f=subprocess.Popen([config.get("path","smilint"),"-m",path],stderr=subprocess.PIPE,env={"SMIPATH":":".join(smipath)}).stderr
        for l in f:
            match=rx_module_not_found.search(l.strip())
            if match:
                raise MIBRequiredException("Uploaded MIB",match.group(1))
        # Convert MIB to python module and load
        h,p=tempfile.mkstemp()
        subprocess.check_call([config.get("path","smidump"),"-k","-f","python","-o",p,path],
            env={"SMIPATH":":".join(smipath)})
        m=imp.load_source("mib",p)
        os.close(h)
        os.unlink(p)
        mib_name=m.MIB["moduleName"]
        # Check module dependencies
        depends_on={}
        if "imports" in m.MIB:
            for i in m.MIB["imports"]:
                if "module" not in i:
                    continue
                rm=i["module"]
                if rm in depends_on:
                    continue
                try:
                    depends_on[rm]=MIB.objects.get(name=rm)
                except MIB.DoesNotExist:
                    raise MIBRequiredException(mib_name,rm)
        # Get MIB latest revision date
        try:
            last_updated=datetime.datetime.strptime(sorted([x["date"] for x in m.MIB[mib_name]["revisions"]])[-1],"%Y-%m-%d %H:%M")
        except:
            last_updated=datetime.datetime(year=1970,month=1,day=1)
        # Check mib already uploaded
        mib_description=m.MIB[mib_name].get("description",None)
        try:
            mib=MIB.objects.get(name=mib_name)
            # Skip same version
            if mib.last_updated>=last_updated:
                return mib
            mib.description=mib_description
            mib.uploaded=datetime.datetime.now()
            mib.last_updated=last_updated
            mib.save()
            # Delete all MIB Data
            [d.delete() for d in mib.mibdata_set.all()]
        except MIB.DoesNotExist:
            o=None
            mib=MIB(name=mib_name,description=mib_description,uploaded=datetime.datetime.now(),last_updated=last_updated)
            mib.save()
        # Save MIB Data
        for i in ["nodes","notifications"]:
            if i in m.MIB:
                for node,v in m.MIB[i].items():
                    try: # Do not import duplicated OIDs
                        MIBData.objects.get(oid=v["oid"])
                    except MIBData.DoesNotExist:
                        d=MIBData(mib=mib,oid=v["oid"],name="%s::%s"%(mib_name,node),description=v.get("description",None))
                        d.save()
        # Save MIB Dependency
        for r in depends_on.values():
            md=MIBDependency(mib=mib,requires_mib=r)
            md.save()
        # Save MIB to cache if not uploaded from cache
        lcd=os.path.join("local","share","mibs")
        if not os.path.isdir(lcd): # Ensure directory exists
            os.makedirs(os.path.join("local","share","mibs")) 
        local_cache_path=os.path.join(lcd,"%s.mib"%mib_name)
        cache_path=os.path.join("share","mibs","%s.mib"%mib_name)
        if (os.path.exists(local_cache_path) and os.path.samefile(path,local_cache_path))\
            or (os.path.exists(cache_path) and os.path.samefile(path,cache_path)):
            return mib
        with open(path) as f:
            data=f.read()
        safe_rewrite(local_cache_path,data)
        return mib
    ##
    ## Get OID by name
    ##
    @classmethod
    def get_oid(cls,name):
        try:
            o=MIBData.objects.get(name=name)
            return o.oid
        except MIBData.DoesNotExist:
            return None
    ##
    ## Get longest matched name by oid
    ##
    @classmethod
    def get_name(cls,oid):
        l_oid=oid.split(".")
        rest=[]
        while l_oid:
            c_oid=".".join(l_oid)
            try:
                o=MIBData.objects.get(oid=c_oid)
                name=o.name
                if rest:
                    name+="."+".".join(reversed(rest))
                return name
            except MIBData.DoesNotExist:
                rest.append(l_oid.pop())
        return oid
##
## MIB elements
##
class MIBData(models.Model):
    class Meta:
        verbose_name="MIB Data"
        verbose_name_plural="MIB Data"
        ordering=["oid"]
    mib = models.ForeignKey(MIB,verbose_name="MIB")
    oid = models.CharField("OID",max_length=128,unique=True)
    name= models.CharField("Name",max_length=128,unique=True)
    description= models.TextField("Description",blank=True,null=True)

    def __unicode__(self):
        return "%s:%s = %s"%(self.mib.name,self.name,self.oid)
##
## MIB Dependency
##
class MIBDependency(models.Model):
    class Meta:
        verbose_name="MIB Import"
        verbose_name_plural="MIB Imports"
        unique_together=[("mib","requires_mib")]
    mib=models.ForeignKey(MIB,verbose_name="MIB")
    requires_mib=models.ForeignKey(MIB,verbose_name="Requires MIB",related_name="requiredbymib_set")
    
    def __unicode__(self):
        return "%s requires %s"%(self.mib.name,self.requires_mib.name)
    ##
    ## Return graphviz dot with MIB dependencies
    ##
    @classmethod
    def get_dot(cls):
        r=["digraph {"]
        r+=["label=\"MIB Dependencies\";"]
        for d in cls.objects.all():
            r+=["\"%s\" -> \"%s\";"%(d.mib.name,d.requires_mib.name)]
        r+=["}"]
        return "\n".join(r)
    ##
    ## Write graphviz dot with MIB dependencies
    ##
    @classmethod
    def write_dot(cls,path):
        safe_rewrite(path,cls.get_dot())
##
## Events
##
class EventPriority(models.Model):
    class Meta:
        verbose_name="Event Priority"
        verbose_name_plural="Event Priorities"
        ordering=["priority"]
    name=models.CharField("Name",max_length=32,unique=True)
    priority=models.IntegerField("Priority")
    description=models.TextField("Description",blank=True,null=True)
    font_color=models.CharField("Font Color",max_length=32,blank=True,null=True)
    background_color=models.CharField("Background Color",max_length=32,blank=True,null=True)
    def __unicode__(self):
        return self.name
    def _css_style_name(self):
        return "CSS_%s"%self.name.replace(" ","")
    css_style_name=property(_css_style_name)
    def _css_style(self):
        s=[]
        if self.font_color:
            s+=["    color: %s;"%self.font_color]
        if self.background_color:
            s+=["    background: %s;"%self.background_color]
        return ".%s {\n%s\n}"%(self.css_style_name,"\n".join(s))
    css_style=property(_css_style)
##
## Event categories.
## Event categories separate events to area of responsibility (Network event, System event, Security event)
##
class EventCategory(models.Model):
    class Meta:
        verbose_name="Event Category"
        verbose_name_plural="Event Categories"
        ordering=["name"]
    name=models.CharField("Name",max_length=32,unique=True)
    description=models.TextField("Description",blank=True,null=True)

    def __unicode__(self):
        return self.name
##
## Event classes.
## Event class specifies a kind of event with predefined set of event variables.
## Event classes are assigned by Classifier process.
## Correlator process performs event corelation mostly using event class analysys
##
class EventClass(models.Model):
    class Meta:
        verbose_name="Event Class"
        verbose_name_plural="Event Classes"
        ordering=["name"]
    name=models.CharField("Name",max_length=64)
    category=models.ForeignKey(EventCategory,verbose_name="Event Category")
    default_priority=models.ForeignKey(EventPriority,verbose_name="Default Priority")
    subject_template=models.CharField("Subject Template",max_length=128)
    body_template=models.TextField("Body Template")
    last_modified=models.DateTimeField("last_modified",auto_now=True)
    repeat_suppression=models.BooleanField("Repeat Suppression",default=False)
    repeat_suppression_interval=models.IntegerField("Repeat Suppression interval (secs)",default=3600)
    trigger=models.CharField("Trigger",max_length=64,null=True,blank=True,choices=event_trigger_registry.choices)
    is_builtin=models.BooleanField("Is Builtin",default=False)
    
    def __unicode__(self):
        return self.name
    
    def python_link(self):
        return "<A HREF='/fm/py_event_class/%d/'>Python</A>"%self.id
    python_link.short_description="Python"
    python_link.allow_tags=True
    ##
    ## Run trigger if defined
    ##
    def run_trigger(self,event):
        if self.trigger and self.trigger in event_trigger_registry.classes:
            event_trigger_registry.classes[self.trigger]().handle(event)
    ##
    ## Python representation of data structure
    ##
    def _python_code(self):
        s=["##","## %s"%self.name,"##"]
        s+=["class %s(EventClass):"%rx_py_id.sub("",self.name)]
        s+=["    name     = \"%s\""%py_q(self.name)]
        s+=["    category = \"%s\""%py_q(self.category.name)]
        s+=["    priority = \"%s\""%py_q(self.default_priority.name)]
        s+=["    subject_template=\"%s\""%py_q(self.subject_template)]
        s+=["    body_template=\"\"\"%s\"\"\""%self.body_template]
        s+=["    repeat_suppression=%s"%self.repeat_suppression]
        s+=["    repeat_suppression_interval=%d"%self.repeat_suppression_interval]
        if self.trigger:
            s+=["    trigger=\"%s\""%self.trigger]
        else:
            s+=["    trigger=None"]
        vars=list(self.eventclassvar_set.all())
        if vars:
            s+=["    class Vars:"]
            for v in vars:
                s+=["        %s=Var(required=%s,repeat=%s)"%(v.name,v.required,v.repeat_suppression)]
        s+=[]
        return "\n".join(s)
    python_code=property(_python_code)
##
## Event class variables
##
class EventClassVar(models.Model):
    class Meta:
        verbose_name="Event Class Variable"
        verbose_name_plural="Event Class Variables"
        unique_together=[("event_class","name")]
    event_class=models.ForeignKey(EventClass,verbose_name="Event Class")
    name=models.CharField("Name",max_length=64)
    required=models.BooleanField("Required",default=True)
    repeat_suppression=models.BooleanField("Repeat Suppression",default=False) # Used for repeat supression
    def __unicode__(self):
        return "%s: %s"%(self.event_class,self.name)
##
## Classification rule.
## Used by Correlator process for assigning event class to unclassified events
##
class EventClassificationRule(models.Model):
    class Meta:
        verbose_name="Event Classification Rule"
        verbose_name_plural="Event Classification Rules"
        ordering=["preference"]
    event_class=models.ForeignKey(EventClass,verbose_name="Event Class")
    name=models.CharField("Name",max_length=64)
    preference=models.IntegerField("Preference",1000)
    action=models.CharField("Action",max_length=1,choices=[("A","Make Active"),("C","Close"),("D","Drop")],default="A")
    is_builtin=models.BooleanField("Is Builtin",default=False)
    
    def __unicode__(self):
        return self.name
        
    def python_link(self):
        return "<A HREF='/fm/py_event_classification_rule/%d/'>Python</A>"%self.id
    python_link.short_description="Python"
    python_link.allow_tags=True
    ##
    ## Python representation of data structure
    ##
    def _python_code(self):
        s=["from noc.fm.rules.classification import ClassificationRule,Expression,CLOSE_EVENT,DROP_EVENT"]
        s+=["##","## %s"%self.name,"##"]
        s+=["class %s_Rule(ClassificationRule):"%rx_py_id.sub("_",self.name)]
        s+=["    name=\"%s\""%py_q(self.name)]
        s+=["    event_class=%s"%self.event_class.name.replace(" ","").replace("-","_")]
        s+=["    preference=%d"%self.preference]
        if self.action!="A":
            s+=["    action=%s"%{"C":"CLOSE_EVENT","D":"DROP_EVENT"}[self.action]]
        s+=["    patterns=["]
        for p in self.eventclassificationre_set.all():
            if p.is_expression:
                s+=["        Expression(r\"%s\",r\"%s\"),"%(py_q(p.left_re),py_q(p.right_re))]
            else:
                s+=["        (r\"%s\",r\"%s\"),"%(py_q(p.left_re),py_q(p.right_re))]
        s+=["    ]",""]
        return "\n".join(s)
    python_code=property(_python_code)
##
## Regular expressions to match event vars
##
class EventClassificationRE(models.Model):
    class Meta:
        verbose_name="Event Classification RE"
        verbose_name_plural="Event Classification REs"
    rule=models.ForeignKey(EventClassificationRule,verbose_name="Event Classification Rule")
    left_re=models.CharField("Left RE",max_length=256)
    right_re=models.CharField("Right RE",max_length=256)
    is_expression=models.BooleanField("Is Expression",default=False)
    ##
    ## Check expression syntax before save
    ##
    def save(self):
        if self.is_expression:
            compile(self.right_re,"inline","eval") # Raise SyntaxError in case of invalid expression
        super(EventClassificationRE,self).save()
##
## Event Post Processing
##
class EventPostProcessingRule(models.Model):
    class Meta:
        verbose_name="Event Post-Processing Rule"
        verbose_name_plural="Event Post-Processing Rules"
    event_class=models.ForeignKey(EventClass,verbose_name="Event Class")
    name=models.CharField("Name",max_length=64)
    preference=models.IntegerField("Preference",default=1000)
    is_active=models.BooleanField("Is Active",default=True)
    description=models.TextField("Description",blank=True,null=True)
    # Actions
    change_priority=models.ForeignKey(EventPriority,verbose_name="Change Priority to",blank=True,null=True)
    change_category=models.ForeignKey(EventCategory,verbose_name="Change Category to",blank=True,null=True)
    action=models.CharField("Action",max_length=1,choices=[("A","Make Active"),("C","Close"),("D","Drop")],default="A")
    def __unicode__(self):
        return self.name
##
## Regular expressions to match event vars
##
class EventPostProcessingRE(models.Model):
    class Meta:
        verbose_name="Event Post-Processing RE"
        verbose_name_plural="Event Post-Processing REs"
    rule=models.ForeignKey(EventPostProcessingRule,verbose_name="Event Post-Processing Rule")
    var_re=models.CharField("Var RE",max_length=256)
    value_re=models.CharField("Value RE",max_length=256)
##
## Event Correlation Rule
## Correlation rules are the PyKE statements
##
class EventCorrelationRule(models.Model):
    class Meta:
        verbose_name="Event Correlation Rule"
        verbose_name_plural="Event Correlation Rules"
        ordering=["name"]
    name=models.CharField("Name",max_length=64,unique=True)
    rule=models.TextField("Rule")
    description=models.TextField("Description",null=True,blank=True)
    is_builtin=models.BooleanField("Is Builtin",default=False)
    def __unicode__(self):
        return self.name
    def _pyke_id(self):
        return rx_py_id.sub(self.name,"_")
    pyke_id=property(_pyke_id)
    def _pyke_code(self):
        s=[]
        if self.description:
            s+=["##"]+["## %s"%l for l in self.description.split("\n")]+["##"]
        s+=[self.pyke_id]
        s+=["    %s"%l for l in self.rule.split("\n")]+[""]
        return "\n".join(s)
    pyke_code=property(_pyke_code)
##
## Event itself
##
EVENT_STATUS_CHOICES=[("U","Unclassified"),("A","Active"),("C","Closed")]
class Event(models.Model):
    class Meta:
        verbose_name="Event"
        verbose_name_plural="Events"
        ordering=[("id")]
    timestamp=models.DateTimeField("Timestamp")
    managed_object=models.ForeignKey(ManagedObject,verbose_name="Managed Object")
    event_priority=models.ForeignKey(EventPriority,verbose_name="Priority")
    event_category=models.ForeignKey(EventCategory,verbose_name="Event Category")
    event_class=models.ForeignKey(EventClass,verbose_name="Event Class")
    status=models.CharField("Status",max_length=1,choices=EVENT_STATUS_CHOICES,default="U")
    close_timestamp=models.DateTimeField("Close Timestamp",blank=True,null=True)
    active_till=models.DateTimeField("Active Till",blank=True,null=True)          # Time-based event closing
    root=models.ForeignKey("Event",verbose_name="Root cause",blank=True,null=True)# Set up by correlator, Null for root cause
    subject=models.CharField("Subject",max_length=256,null=True,blank=True)       # Not null when classified
    body=models.TextField("Body",null=True,blank=True)
    
    def __unicode__(self):
        return u"Event #%s: %s"%(str(self.id),str(self.subject))
    
    def _repeats(self):
        return self.eventrepeat_set.count()
    repeats=property(_repeats)
    
    def _data(self):
        r={}
        for d in self.eventdata_set.all():
            r[d.key]=d.value
        return r
    data=property(_data)
    
    def match_data(self,vars):
        data=self.data
        for k,v in vars.items():
            if k not in data or data[k]!=v:
                return False
        return True
    ##
    ## Reset event status to "Unclassified"
    ##
    def reclassify_event(self,message="Reclassification requested"):
        self.change_status("U",message)
    ##
    ## Close event
    ##
    def close_event(self,message="Event Closed"):
        self.change_status("C",message)
    ##
    ##
    ##
    def open_event(self,message="Event Opened"):
        self.change_status("A",message)
    ##
    ## Change event status
    ##
    def change_status(self,status,msg=None):
        if self.status==status:
            return
        if msg is None:
            msg="Status changed from %s to %s"%(self.status,status)
        from_status=self.status
        self.status=status
        self.save()
        self.log(msg,from_status,status)
    ##
    ## Log event processing message
    ##
    def log(self,msg,from_status=None,to_status=None):
        if from_status is None:
            from_status=self.status
        if to_status is None:
            to_status=self.status
        l=EventLog(event=self,timestamp=datetime.datetime.now(),from_status=from_status,to_status=to_status,message=msg)
        l.save()
##
## Event body
##
class EventData(models.Model):
    class Meta:
        verbose_name="Event Data"
        verbose_name_plural="Event Data"
        unique_together=[("event","key","type")]
    event=models.ForeignKey(Event,verbose_name="Event")
    key=models.CharField("Key",max_length=256)
    value=models.TextField("Value",blank=True,null=True)
    type=models.CharField("Type",max_length=1,choices=[(">","Received"),("V","Variable"),("R","Resolved")],default=">")
##
## Repeated events are deleated from Event table and only short record in EventRepeat remains
##
class EventRepeat(models.Model):
    class Meta:
        verbose_name="Event Repeat"
        verbose_name_plural="Event Repeats"
    event=models.ForeignKey(Event,verbose_name="Event")
    timestamp=models.DateTimeField("Timestamp")
##
## Event Processing Log
##
class EventLog(models.Model):
    class Meta:
        verbose_name="Event Log"
        verbose_name="Event Logs"
        ordering=["event","timestamp"]
    event=models.ForeignKey(Event,verbose_name="Event")
    timestamp=models.DateTimeField("Timestamp")
    from_status=models.CharField("From Status",max_length=1,choices=EVENT_STATUS_CHOICES)
    to_status=models.CharField("To Status",max_length=1,choices=EVENT_STATUS_CHOICES)
    message=models.TextField("Message")
##
## Application Menu
##
class AppMenu(Menu):
    app="fm"
    title="Fault Management"
    items=[
        ("Events", "/fm/", "fm.change_event"),
        ("Setup", [
            ("MIBs",                "/admin/fm/mib/",                     "fm.change_mib"),
            ("MIB Data",            "/admin/fm/mibdata/",                 "fm.change_mibdata"),
            ("Event Classes",       "/admin/fm/eventclass/",              "fm.change_eventclass"),
            ("Event Categories",    "/admin/fm/eventcategory/",           "fm.change_eventcategory"),
            ("Event Priorities",    "/admin/fm/eventpriority/",           "fm.change_eventpriority"),
            ("Classification Rules","/admin/fm/eventclassificationrule/", "fm.change_eventclassificationrule"),
            ("Post-Processing Rules", "/admin/fm/eventpostprocessingrule/", "fm.change_eventpostprocessingrule"),
            ("Correlation Rules",   "/admin/fm/eventcorrelationrule/",    "fm.change_eventcorrelationrule"),
        ])
    ]
